--------------------------------------------------------
-- Archivo creado  - lunes-abril-20-2015   
--------------------------------------------------------
--------------------------------------------------------
--  DDL for Package Body UTILIDADES
--------------------------------------------------------

 create or replace PACKAGE BODY  "UTILIDADES" AS

/* ****************************************************
@Nombre:		Es_Letra
@Funcionalidad:	Funcin que comprueba que el carcter introducido 
				sea un carcter alfabtico . 
@restricciones:	Ninguna.
@params IN:		str IN VARCHAR2
@params OUT:	BOOLEAN
@Observaciones	Utilizaremos la funcin integrada de PL/SQL  TO_NUMBER, 
				para determinar la validez.
**************************************************** */
FUNCTION Es_Letra(str IN VARCHAR2) RETURN BOOLEAN AS
  N NUMBER;
BEGIN
	N := TO_NUMBER(str);
	RETURN (FALSE);

	Exception WHEN OTHERS THEN
	RETURN (TRUE);
END;

/* ****************************************************
@Nombre:		Es_Numero
@Funcionalidad:	Funcin que comprueba que el carcter introducido 
				sea un carcter numrico. 
@restricciones:	Ninguna.
@params IN:		str IN VARCHAR2
@params OUT:	BOOLEAN
@Observaciones	Utilizaremos la funcin integrada de PL/SQL  TO_NUMBER, 
				para determinar la validez.
**************************************************** */
FUNCTION Es_Numero(str IN VARCHAR2) RETURN BOOLEAN AS
  N NUMBER;
BEGIN
	N := TO_NUMBER(str);
	RETURN (TRUE);

	Exception WHEN OTHERS THEN
	RETURN (FALSE);

END;

/* ****************************************************
@Nombre:		Es_Fecha
@Funcionalidad:	Funcin que comprueba que el carcter introducido 
				sea una fecha. 
@restricciones:	Ninguna.
@params IN:		p_str IN VARCHAR2
@params OUT:	BOOLEAN
@Observaciones	Utilizaremos la funcin integrada de PL/SQL  TO_DATE, 
				para determinar la validez.
**************************************************** */
FUNCTION Es_Fecha( p_str in varchar2 ) RETURN BOOLEAN AS

fecha DATE;
BEGIN
    fecha:= TO_DATE( p_str );
    RETURN TRUE;
EXCEPTION
    WHEN OTHERS THEN
    RETURN FALSE;
END;

/* ****************************************************
@Nombre:		Validar_Email
@Funcionalidad:	Funcin que comprueba la validez formal de un email.  
@restricciones:	Ninguna.
@params IN:		email IN VARCHAR2
@params OUT:	BOOLEAN
@Observaciones	No se comprueba la existencia del email. 
**************************************************** */
FUNCTION Validar_Email(email IN VARCHAR2) RETURN BOOLEAN AS
  punto_pos    	NUMBER;
  arroba_pos    NUMBER;
  str_longuitud NUMBER;
BEGIN
  punto_pos    	:= INSTR(email ,'.');
  arroba_pos    := INSTR(email ,'@');
  str_longuitud := length(email);

  IF ((punto_pos = 0) OR (arroba_pos = 0) OR (punto_pos = arroba_pos + 1) OR
     (arroba_pos = 1) OR (arroba_pos = str_longuitud) OR
     (punto_pos = str_longuitud))
  THEN
    RETURN FALSE;
  END IF;

  IF INSTR(substr(email ,arroba_pos) ,'.') = 0  THEN
    RETURN FALSE;
  END IF;

  RETURN TRUE;

END validar_Email;

/* ****************************************************
@Nombre:		ValidaNIFCIF
@Funcionalidad:	Funcin que comprueba una identificacin fiscal. 
@restricciones:	Ninguna.
@params IN:		NIFCIF VARCHAR2
@params OUT:	BOOLEAN
@Observaciones	Se validan NIF, DNI, NIE. 
**************************************************** */
FUNCTION ValidaNIFCIF(NIFCIF VARCHAR2) RETURN BOOLEAN AS
    cControl1 	VARCHAR2(23);
	cControl2 	VARCHAR2(10);
	cNIF 	  	VARCHAR2(8);
	nSubNIF 	VARCHAR2(1);
	cCter 		VARCHAR2(1);
	kLetra 		VARCHAR2(1);
	cIni 		VARCHAR2(1);
	cFin 		VARCHAR2(1);

	nResulta 	NUMBER;
	nResul1 	NUMBER;
	nResul2 	NUMBER;
	nDigito 	NUMBER;
	nDControl 	NUMBER;
	nR2 		NUMBER;
	nTr2 		NUMBER;
	nR4 		NUMBER;
	nTr4 		NUMBER;
	nR6 		NUMBER;
	nTr6 		NUMBER;
	nR8 		NUMBER;
	nTr8 		NUMBER;
	cR2A 		NUMBER;
	cR2B 		NUMBER;
	cR2L 		NUMBER;
	cR4A 		NUMBER;
	cR4B 		NUMBER;
	cR4L 		NUMBER;
	cR6A 		NUMBER;
	cR6B 		NUMBER;
	cR6L 		NUMBER;
	cR8A 		NUMBER;
	cR8B 		NUMBER;
	cR8L 		NUMBER;
	nNIF 		NUMBER;

    BEGIN
        IF (LENGTH(NIFCIF) < 8) THEN
            RETURN FALSE;
        END IF;

        IF NOT UTILIDADES.ES_Numero(SUBSTR (NIFCIF, 2, 7)) THEN
           RETURN FALSE;
        END IF;


       /* ---------Comprobacin matemtica-------------------- */
        cControl1 := 'TRWAGMYFPDXBNJZSQVHLCKE';
        cControl2 := 'ABCDEFGHIJ';

        cNIF := SUBSTR (NIFCIF, 1, 8);
        cCter := SUBSTR (NIFCIF, 9, 1);
        cIni := SUBSTR (cNIF, 1,1);
        cFin := SUBSTR (NIFCIF, 9, 1);
        IF INSTR('0123456789', SUBSTR (cNIF, 1, 1)) > 0 THEN
            nNIF := TO_NUMBER(cNIF);
            nResulta := (nNIF MOD 23) + 1;
            kLetra := SUBSTR (cControl1, nResulta, 1);
            IF kLetra = cCter THEN
                RETURN TRUE;
            Else
                RETURN FALSE;
            END IF;
        Else
            IF cIni = 'K' OR 
			   cIni = 'L' OR 
			   cIni = 'X' OR 
			   cIni = 'Y' THEN
                IF cIni = 'Y' THEN
                    nSubNIF := '1' || (SUBSTR (cNIF, 2, 7));
                Else
                    nSubNIF := TO_NUMBER(SUBSTR (cNIF, 2, 7));
                END IF;
                nResulta := (nSubNIF Mod 23) + 1;
                kLetra := SUBSTR (cControl1, nResulta, 1);
                IF kLetra = cCter THEN
                    RETURN TRUE;
                Else
                    RETURN FALSE;
                END IF;

            Else

               /* Puede que en un futuro lleguen las letras W, J, U, R, V, y 
			   se tendra que verificar que estructura siguen */
                IF (cIni >= 'A' And cIni <= 'H') OR cIni = 'P' OR 
				   cIni = 'Q' OR cIni = 'S' OR
                   cIni = 'N' OR cIni = 'J' OR cIni = 'R' OR 
				   cIni = 'U' OR cIni = 'V' OR cIni = 'W' THEN
                    nResul1 := TO_NUMBER(SUBSTR (cNIF, 3, 1)) + 
							   TO_NUMBER(SUBSTR (cNIF, 5, 1)) + 
							   TO_NUMBER(SUBSTR (cNIF, 7, 1));
                    nR2 := TO_NUMBER(SUBSTR (cNIF, 2, 1)) * 2;
                    nR4 := TO_NUMBER(SUBSTR (cNIF, 4, 1)) * 2;
                    nR6 := TO_NUMBER(SUBSTR (cNIF, 6, 1)) * 2;
                    nR8 := TO_NUMBER(SUBSTR (cNIF, 8, 1)) * 2;
                    IF LENGTH(TRIM(TO_CHAR(nR2))) > 1 THEN
                        cR2L := TRIM(TO_CHAR(nR2));
                        cR2A := SUBSTR (cR2L, 1, 1);
                        cR2B := SUBSTR (cR2L, 2, 1);
                        nTr2 := TO_NUMBER(cR2A) + TO_NUMBER(cR2B);
                    Else
                        nTr2 := nR2;
                    END IF;

                    IF LENGTH(TRIM(TO_CHAR(nR4))) > 1 THEN
                        cR4L := TRIM(TO_CHAR(nR4));
                        cR4A := SUBSTR (cR4L, 1, 1);
                        cR4B := SUBSTR (cR4L, 2, 1);
                        nTr4 := TO_NUMBER(cR4A) + TO_NUMBER(cR4B);
                    Else
                        nTr4 := nR4;
                    END IF;

                    IF LENGTH(TRIM(TO_CHAR(nR6))) > 1 THEN
                        cR6L := TRIM(TO_CHAR(nR6));
                        cR6A := SUBSTR (cR6L, 1, 1);
                        cR6B := SUBSTR (cR6L, 2, 1);
                        nTr6 := TO_NUMBER(cR6A) + TO_NUMBER(cR6B);
                    Else
                        nTr6 := nR6;
                    END IF;

                    IF LENGTH(TRIM(TO_CHAR(nR8))) > 1 THEN
                        cR8L := TRIM(TO_CHAR(nR8));
                        cR8A := SUBSTR (cR8L, 1, 1);
                        cR8B := SUBSTR (cR8L, 2, 1);
                        nTr8 := TO_NUMBER(cR8A) + TO_NUMBER(cR8B);
                    Else
                        nTr8 := nR8;
                    END IF;

                    nResul2 := nTr2 + nTr4 + nTr6 + nTr8;
                    nResulta := nResul1 + nResul2;
                    nDigito := nResulta Mod 10;
                    nDControl := 10 - nDigito;
                    IF INSTR('PQS', cIni) > 0 THEN
                        IF SUBSTR (cControl2, nDControl, 1) = cCter THEN
                            RETURN TRUE;
                        Else
                            RETURN FALSE;
                        END IF;
                    Else
                        IF INSTR('ABCDEFGHIJ', cFin) > 0 THEN
                            IF SUBSTR (cControl2, nDControl, 1) = cCter THEN
                                RETURN TRUE;
                            Else
                                RETURN FALSE;
                            END IF;
                        Else
                            IF nDControl = 10 THEN
                               nDControl := 0;
                            END IF;
                            IF TRIM(TO_CHAR(nDControl)) = cCter THEN
                                RETURN TRUE;
                            Else
                                RETURN FALSE;
                            END IF;
                        END IF;

                    END IF;
                Else
                    RETURN FALSE;
                END IF;
            END IF;

		END IF;
    END;

/* ****************************************************
@Nombre:		Revalorizacion_Accion_Anual
@Funcionalidad:	Funcin en la que dada una accin y un ao, 
				nos devuelve el porcentaje de revalorizacin. 
@restricciones:	Ninguna.
@params IN:		p_str in CHAR, p_year in NUMBER
@params OUT:	porcentaje out NUMBER
**************************************************** */	
FUNCTION Revalorizacion_Accion_Anual(p_str in CHAR, p_year in NUMBER) RETURN NUMBER AS    
	porcentaje NUMBER;
	existe NUMBER;
	fechaMaxima DATE;
	fechaMinima DATE;
	impMax NUMBER;
	impMin NUMBER;
  
BEGIN
	SELECT  Count(*) , MAX(FechaCierre) , MIN(FechaCierre) INTO
            existe, fechaMaxima, fechaMinima
	FROM    AccionHistorico
	WHERE EXTRACT(YEAR FROM FechaCierre) = p_year AND   
        (ISINId = p_str);
  
	IF ( existe = 0 ) THEN
		porcentaje := 0; /* no existe */
	ELSIF ( existe = 1 ) THEN
		porcentaje := 100; /* tan solo existe un apunte */
	ELSE
       SELECT ValorCierre INTO impMax 
       FROM AccionHistorico
       WHERE  FechaCierre = fechaMaxima AND   
             (ISINId = p_str);
             
       SELECT ValorCierre INTO impMin 
       FROM AccionHistorico
       WHERE  FechaCierre = fechaMinima AND   
             (ISINId = p_str);   
    END IF;         
             
	IF impMin = 0 THEN
		porcentaje := 0;
	ELSE
		porcentaje := (impMax- impMin) * 100/ impMin;
	END IF;
  
RETURN porcentaje;
END;

END UTILIDADES;
/
