--------------------------------------------------------
-- Archivo creado  - domingo-abril-19-2015   
--------------------------------------------------------
--------------------------------------------------------
--  DDL for Package TablasAuxiliares
--------------------------------------------------------

CREATE OR REPLACE
PACKAGE BODY TABLASAUXILIARES AS

 existe	NUMBER;
 
  parametrosEntrada		VARCHAR2(1024);
  nombreProcedimento    VARCHAR2(50);
  parametrosSalida      VARCHAR2(255);
   
  ERR_NUM	NUMBER(6);
  ERR_NN	EXCEPTION;

  MSG_ERR	VARCHAR2(96);
  MSG_NN 	VARCHAR2(96);
  MSG_OK 	VARCHAR2(96);

/* ****************************************************
PROCEDIMENTO 	Alta Mercado
@Nombre:		ins_Mercado
@Funcionalidad:	Tabla auxiliar que contendrá el identificador 
				y la descripción de los Mercados de Valores
@restricciones:	Ninguna.
@params IN:		paramMercadoId CHAR(15), 
				paramMercadoDescripcion VARCHAR2(50)
@params OUT:	parametrosEntrada VARCHAR2(1024),
				nombreProcedimento VARCHAR2(50);
				parametrosSalida VARCHAR2(255);
				RSP
@error			MSG_ERR con la información del error
**************************************************** */
PROCEDURE ins_Mercado
(
  paramMercadoId 			IN Mercado.MercadoId%TYPE, 
  paramMercadoDescripcion 	IN Mercado.MercadoDescripcion%TYPE, 
  RSP						OUT NOCOPY VARCHAR2
) AS

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_Mercado'; 
	parametrosEntrada := 'Mercado: '|| paramMercadoId || ' - '|| 
									paramMercadoDescripcion ;

-- comprobar los datos de entrada  
	IF (paramMercadoId  IS NULL) THEN
		MSG_NN := 'Falta Identificador de Mercado';
		RAISE ERR_NN;
	END IF;
	IF (paramMercadoDescripcion IS NULL) THEN
		MSG_NN := 'Falta la descripción del Mercado';
		RAISE ERR_NN;	
	END IF;	
-- comprobar si el paramMercadoId existe en la BD:   
	SELECT count(*) INTO existe 
	FROM Mercado
	WHERE MercadoId = UPPER(paramMercadoId);

	IF (existe > 0) THEN
		MSG_NN := 'El Mercado ' || paramMercadoId || ' existe en la BD.';
		RAISE ERR_NN;
	END IF;

-- INSERT del nuevo Mercado
	INSERT INTO Mercado(MercadoId, MercadoDescripcion) VALUES 
				(paramMercadoId, paramMercadoDescripcion);

	RSP:='OK';
	MSG_OK := 'Alta Mercado realizada correctamente.';

	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
END ins_Mercado ;

/* ****************************************************
PROCEDIMENTO 		Alta Accion
@Nombre:			ins_Accion
@Funcionalidad:		Tabla auxiliar que contendrá los datos de una Acción.
@restricciones:		Ninguna.
@params IN:			paramISINId CHAR(12), paramTicker VARCHAR2(10), 
					paramTickerDescripcion VARCHAR2(100),paramMercadoId(15)
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50);
					parametrosSalida VARCHAR2(255);
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE ins_Accion
(
  paramISINId 				IN Accion.ISINId%TYPE, 
  paramTicker   	 		IN Accion.Ticker%TYPE, 
  paramTickerDescripcion 	IN Accion.TickerDescripcion%TYPE, 
  paramMercadoId		 	IN Accion.MercadoId%TYPE, 
  RSP						OUT NOCOPY VARCHAR2
) AS

  
BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_Accion'; 
	parametrosEntrada := 'Accion: '|| paramISINId || ' - '|| paramTicker|| ' - '|| 
						paramTickerDescripcion || ' - '|| paramMercadoId ;

-- comprobar los datos de entrada  
	IF (paramISINId  IS NULL) THEN
		MSG_NN := 'Falta Identificador ISIN de la Accion';
		RAISE ERR_NN;
	END IF;
	IF (paramTicker IS NULL) THEN
		MSG_NN := 'Falta Identificador Ticker de la Accion';
		RAISE ERR_NN;	
	END IF;
	IF (paramTickerDescripcion IS NULL) THEN
		MSG_NN := 'Falta la descripción de la Accion';
		RAISE ERR_NN;	
	END IF;	
-- comprobar si el paramMercadoId existe en la BD:   
	SELECT count(*) INTO existe 
	FROM Mercado
	WHERE MercadoId = UPPER(paramMercadoId);

	IF (existe = 0) THEN
		MSG_NN := 'El Mercado ' || paramMercadoId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;
	
-- comprobar si el paramISINId existe en la BD:   
	SELECT count(*) INTO existe 
	FROM Accion
	WHERE ISINId = UPPER(paramISINId);

	IF (existe > 0) THEN
		MSG_NN := 'La acción ' || paramISINId || ' existe en la BD.';
		RAISE ERR_NN;
	END IF;

-- INSERT del nuevo Accion
	INSERT INTO Accion (ISINId,Ticker, TickerDescripcion, MercadoId ,
						NumeroAccionesDisponibles , NumeroClientesPoseedores, 
						VolumenNegociacion) 
	VALUES (paramISINId, paramTicker, paramTickerDescripcion, paramMercadoId, 0,0,0);

	RSP:='OK';
	MSG_OK := 'Alta Acción realizada correctamente.';

	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
	  
END ins_Accion ;

/* ****************************************************
PROCEDIMENTO 	Alta TipoOrden
@Nombre:		ins_TipoOrden
@Funcionalidad:	Tabla auxiliar que contendrá el identificador y 
				la descripción de los TipoOrdenes
@restricciones:	Ninguna.
@params IN:		paramTipoOrdenId CHAR(1), 
				paramTipoOrdenDescripcion VARCHAR2(15),
				paramSignoTipoOrden SMALLINT
@params OUT:	parametrosEntrada VARCHAR2(1024),
				nombreProcedimento VARCHAR2(50);
				parametrosSalida VARCHAR2(255);
				RSP
@error			MSG_ERR con la información del error
**************************************************** */
PROCEDURE ins_TipoOrden
(
  paramTipoOrdenId 				IN TipoOrden.TipoOrdenId%TYPE, 
  paramTipoOrdenDescripcion 	IN TipoOrden.TipoOrdenDescripcion%TYPE, 
  paramSignoTipoOrden 		    IN TipoOrden.SignoTipoOrden%TYPE, 
  RSP							OUT NOCOPY VARCHAR2
) AS

  
BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_TipoOrden'; 
	parametrosEntrada := 'TipoOrden: '|| paramTipoOrdenId || ' - '|| 
										paramTipoOrdenDescripcion || ' - '|| 
										paramSignoTipoOrden ;

-- comprobar los datos de entrada  
	IF (paramTipoOrdenId  IS NULL) THEN
		MSG_NN := 'Falta Identificador de TipoOrden';
		RAISE ERR_NN;
	END IF;
	IF (paramTipoOrdenDescripcion IS NULL) THEN
		MSG_NN := 'Falta la descripción del TipoOrden';
		RAISE ERR_NN;	
	END IF;	
	IF (paramSignoTipoOrden IS NULL) THEN
		MSG_NN := 'Falta asignar el signo ';
		RAISE ERR_NN;	
	END IF;	
	IF (paramSignoTipoOrden != 1) and  (paramSignoTipoOrden != -1) THEN
		MSG_NN := 'Valor signo incorrecto (debe ser 1 o -1)';
		RAISE ERR_NN;	
	END IF;	
	
	-- comprobar paramTipoOrdenId no exista en la BD.   
	SELECT count(*) INTO existe 
	FROM TipoOrden
	WHERE TipoOrdenId = UPPER(paramTipoOrdenId);

	IF (existe > 0) THEN
		MSG_NN := 'Tipo Orden ' || paramTipoOrdenId || ' existe en la BD.';
		RAISE ERR_NN;
	END IF;
	
-- INSERT del nuevo TipoOrden
	INSERT INTO TipoOrden (TipoOrdenId, TipoOrdenDescripcion, SignoTipoOrden)
	VALUES (paramTipoOrdenId, paramTipoOrdenDescripcion, paramSignoTipoOrden);

	RSP:='OK';
	MSG_OK := 'Alta TipoOrden realizada correctamente.';

	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);

END ins_TipoOrden ;

/* ****************************************************
PROCEDIMENTO 	Alta OrdenEstado
@Nombre:		ins_OrdenEstado
@Funcionalidad:	Tabla auxiliar que contendrá el identificador y 
				la descripción de las OrdenEstado
@restricciones:	Ninguna.
@params IN:		paramEstadoId CHAR(1), 
				paramEstadoDescripcion VARCHAR2(25)
@params OUT:	parametrosEntrada VARCHAR2(1024),
				nombreProcedimento VARCHAR2(50);
				parametrosSalida VARCHAR2(255);
				RSP
@error			MSG_ERR con la información del error
**************************************************** */
PROCEDURE ins_OrdenEstado
(
  paramEstadoId 				IN OrdenEstado.EstadoId%TYPE, 
  paramEstadoOrdenDescripcion 	IN OrdenEstado.EstadoOrdenDescripcion%TYPE, 
  RSP							OUT NOCOPY VARCHAR2
) AS

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_OrdenEstado'; 
	parametrosEntrada := 'OrdenEstado: '|| paramEstadoId || ' - '|| 
										paramEstadoOrdenDescripcion  ;

-- comprobar los datos de entrada  
	IF (paramEstadoId  IS NULL) THEN
		MSG_NN := 'Falta Identificador de OrdenEstado';
		RAISE ERR_NN;
	END IF;
	IF (paramEstadoOrdenDescripcion IS NULL) THEN
		MSG_NN := 'Falta la descripción del OrdenEstado';
		RAISE ERR_NN;	
	END IF;	
-- comprobar paramEstadoId no exista en la BD.    
	SELECT count(*) INTO existe 
	FROM OrdenEstado
	WHERE EstadoId = UPPER(paramEstadoId);

	IF (existe > 0) THEN
		MSG_NN := 'Estado ' || paramEstadoId || ' existe en la BD.';
		RAISE ERR_NN;
	END IF;
	
-- INSERT nuevo OrdenEstado
	INSERT INTO OrdenEstado (EstadoId, EstadoOrdenDescripcion)
	VALUES (paramEstadoId, paramEstadoOrdenDescripcion);

	RSP:='OK';
	MSG_OK := 'Alta OrdenEstado realizada correctamente.';
	
	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);

END ins_OrdenEstado ;

/* ****************************************************
PROCEDIMENTO 		Alta Trading
@Nombre:			ins_Trading
@Funcionalidad:		Tabla auxiliar que contendrá el identificador y 
					la descripción de los Tradings
@restricciones:		Ninguna.
@params IN:			paramTradingId CHAR(10), 
					paramTradingDescripcion VARCHAR2(100)
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE ins_Trading
(
  paramTradingId 			IN Trading.TradingId%TYPE, 
  paramTradingDescripcion 	IN Trading.TradingDescripcion%TYPE, 
  RSP						OUT NOCOPY VARCHAR2
) AS

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_Trading'; 
	parametrosEntrada := 'Trading: '|| paramTradingId || ' - '|| 
									paramTradingDescripcion  ;

-- comprobar los datos de entrada  
	IF (paramTradingId  IS NULL) THEN
		MSG_NN := 'Falta Identificador de Trading';
		RAISE ERR_NN;
	END IF;
	IF (paramTradingDescripcion IS NULL) THEN
		MSG_NN := 'Falta la descripción del Trading';
		RAISE ERR_NN;	
	END IF;
	
-- comprobar paramTradingId no exista en la BD.    
	SELECT count(*) INTO existe 
	FROM Trading
	WHERE TradingId = UPPER(paramTradingId);

	IF (existe > 0) THEN
		MSG_NN := 'Trading ' || paramTradingId || ' existe en la BD.';
		RAISE ERR_NN;
	END IF;
		
-- INSERT nuevo Trading
	INSERT INTO Trading (TradingId, TradingDescripcion)
	VALUES (paramTradingId, paramTradingDescripcion);

	RSP:='OK';
	MSG_OK := 'Alta Trading realizada correctamente.';
	
	 -- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
     
END ins_Trading ;

/* ****************************************************
PROCEDIMENTO Alta TipoOrigen
@Nombre:			ins_TipoOrigen
@Funcionalidad:		Tabla auxiliar que contendrá el identificador y 
					la descripción de los TipoOrigene
@restricciones:		Ninguna.
@params IN:			paramTipoOrigenId CHAR(1), 
					paramTipoOrigenDescripcion VARCHAR2(15)
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE ins_TipoOrigen
(
  paramTipoOrigenId 			IN TipoOrigen.TipoOrigenId%TYPE, 
  paramTipoOrigenDescripcion 	IN TipoOrigen.TipoOrigenDescripcion%TYPE, 
  RSP							OUT NOCOPY VARCHAR2
) AS

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_TipoOrigen'; 
	parametrosEntrada := 'TipoOrigen: '|| paramTipoOrigenId || ' - '|| 
										paramTipoOrigenDescripcion  ;

-- comprobar los datos de entrada  
	IF (paramTipoOrigenId  IS NULL) THEN
		MSG_NN := 'Falta Identificador de TipoOrigen';
		RAISE ERR_NN;
	END IF;
	IF (paramTipoOrigenDescripcion IS NULL) THEN
		MSG_NN := 'Falta la descripción del TipoOrigen';
		RAISE ERR_NN;	
	END IF;	
	
-- comprobar paramTipoOrigenId no exista en la BD.   
	SELECT count(*) INTO existe 
	FROM TipoOrigen
	WHERE TipoOrigenId = UPPER(paramTipoOrigenId);

	IF (existe > 0) THEN
		MSG_NN := 'Tipo Origen ' || paramTipoOrigenId || ' existe en la BD.';
		RAISE ERR_NN;
	END IF;
	
-- INSERT nuevo TipoOrigen
	INSERT INTO TipoOrigen (TipoOrigenId, TipoOrigenDescripcion)
	VALUES (paramTipoOrigenId, paramTipoOrigenDescripcion);

	RSP:='OK';
	MSG_OK := 'Alta TipoOrigen realizada correctamente.';

	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);

END ins_TipoOrigen ;

END TABLASAUXILIARES;

/


