--------------------------------------------------------
-- Archivo creado  - domingo-abril-19-2015   
--------------------------------------------------------
--------------------------------------------------------
--  DDL for Package TablasMantenimiento
--------------------------------------------------------

CREATE OR REPLACE
PACKAGE BODY TABLASMANTENIMIENTO AS

  existe	NUMBER;
 
  parametrosEntrada		VARCHAR2(1024);
  nombreProcedimento    VARCHAR2(50);
  parametrosSalida      VARCHAR2(255);
   
  ERR_NUM	NUMBER(6);
  ERR_NN	EXCEPTION;

  MSG_ERR	VARCHAR2(96);
  MSG_NN 	VARCHAR2(96);
  MSG_OK 	VARCHAR2(96);

/* ****************************************************
PROCEDIMENTO 		Alta Cliente
@Nombre:			ins_Cliente
@Funcionalidad:		Alta Tabla de Clientes.
@restricciones:		Ninguna.
@params IN:			paramNombre VARCHAR2(150) , 
					paramNIF CHAR(9), 
					paramEmail VARCHAR2(255)
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE ins_Cliente
(
  paramNombre			IN Cliente.Nombre%TYPE, 
  paramNIF				IN Cliente.NIF%TYPE,
  paramEmail  			IN Cliente.Email%TYPE,
  RSP					OUT NOCOPY VARCHAR2
) AS

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_Cliente'; 
	parametrosEntrada := 'Cliente: ' || paramNombre || ' - '|| 
										paramNIF || ' - '|| 
										paramEmail ;

-- comprobar los datos de entrada  
	IF (paramNombre  IS NULL) THEN
		MSG_NN := 'Falta El Nombre';
		RAISE ERR_NN;
	END IF;
	
	IF (paramNIF IS NULL) THEN
		MSG_NN := 'Falta el NIF';
		RAISE ERR_NN;	
	ELSE
	/* Validar que el identificador fiscal sea correcto. */	
		IF NOT (Utilidades.ValidaNIFCIF(paramNIF)) THEN
			MSG_NN := 'NIF erroneo';
			RAISE ERR_NN;
		END IF;
	END IF;	
	
/* Validar que la dirección Email tenga un formato correcto.
   No se verifica la existencia de la dirección Email. */
	IF (paramEmail IS NOT NULL) THEN
		IF NOT (Utilidades.Validar_Email(paramEmail)) THEN
			MSG_NN := 'Email erroneo';
			RAISE ERR_NN;
		END IF;
	END IF;
	
-- INSERT del nuevo Cliente
	INSERT INTO Cliente (Nombre, NIF, Email,ValorCarteraInversiones, EstaActivo)
	VALUES (paramNombre, paramNIF, paramEmail, 0, 'A');

	RSP:='OK';
	MSG_OK := 'Alta Cliente realizada correctamente.';
	
	 -- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
	  
END ins_Cliente ;

/* ****************************************************
PROCEDIMENTO 		Modificación Cliente
@Nombre:			upd_Cliente
@Funcionalidad:		Modificación Tabla de Clientes.
@restricciones:		Que exita el cliente.
@params IN:			paramClienteiD NUMBER, 
					paramNombre VARCHAR2(150) , 
					paramNIF CHAR(9), 
					paramEmail VARCHAR2(255)
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE upd_Cliente 
(
  paramClienteiD		    IN Cliente.ClienteId%TYPE, 
  paramNombre				IN Cliente.Nombre%TYPE, 
  paramNIF					IN Cliente.NIF%TYPE, 
  paramEmail  				IN Cliente.Email%TYPE,
  RSP						OUT NOCOPY VARCHAR2
) AS
  
  oldNombre       		Cliente.Nombre%TYPE;
  oldNIF       			Cliente.NIF%TYPE;
  oldEmail	     		Cliente.Email%TYPE;
  vNombre       		Cliente.Nombre%TYPE;
  vNIF       			Cliente.NIF%TYPE;
  vEmail	     		Cliente.Email%TYPE;
   
BEGIN
	RSP := 'KO';
	nombreProcedimento := 'upd_Cliente'; 
	parametrosEntrada := 'Cliente: '|| paramClienteiD || ' - '|| 
									   paramNombre || ' - '|| 
									   paramNIF || ' - '|| 
									   paramEmail ;

	-- comprobar los datos de entrada  
	IF (paramClienteiD  IS NULL) THEN
		MSG_NN := 'Falta Identificador del Cliente';
		RAISE ERR_NN;
	END IF;
	
	-- comprobar que  existe en la BD:   
	SELECT count(*) INTO existe
	FROM Cliente
	WHERE ClienteId = paramClienteId;

	IF (existe = 0) THEN
		MSG_NN := 'El Cliente ' || 
					paramClienteId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;
	
/* Selección de los datos del cliente */  
	SELECT 	 Nombre, NIF, Email 
			INTO  oldNombre, oldNIF, oldEmail
	FROM Cliente
	WHERE ClienteId = paramClienteId 
	GROUP BY Nombre, NIF, Email  ;
		
-- comprobar los datos de entrada  
	IF (paramNombre IS NULL) THEN
		vNombre := oldNombre;
	ELSE
		vNombre := paramNombre;
	END IF;	
	
	IF (paramNIF IS NULL) THEN
		vNIF := oldNIF;
	ELSE
	/* Validar que el identificador fiscal sea correcto. */	
		IF NOT (Utilidades.ValidaNIFCIF(paramNIF)) THEN
			MSG_NN := 'NIF erroneo';
			RAISE ERR_NN;
		END IF;
		vNIF := paramNIF;
	END IF;	
	
	IF (paramEmail IS NULL) THEN
		vEmail := oldEmail;
	ELSE
	/* Validar que la dirección Email tenga un formato correcto.
   No se verifica la existencia de la dirección Email. */
		IF NOT (Utilidades.Validar_Email(paramEmail)) THEN
			MSG_NN := 'Email erroneo';
			RAISE ERR_NN;
		END IF;
		vEmail := paramEmail;
	END IF;	
	
	
	
-- Modificación del Cliente
	UPDATE Cliente SET
		Nombre = vNombre,
		NIF = vNIF,
		Email = vEmail
	WHERE ClienteId = paramClienteId;
RSP:='OK';
MSG_OK := 'Modificación Cliente realizada correctamente.';

 -- Gravamos el LOG
  CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
	  
  END upd_Cliente ;

/* ****************************************************
PROCEDIMENTO 		Baja Cliente
@Nombre:			del_Cliente
@Funcionalidad:		Inactivar el cliente. 
					Se realiza una baja lógica, modificando 
					el estado a inactivo. 
@restricciones:		Que exita el cliente.
					Que el valor de sus carteras sea = 0
@params IN:			paramClienteiD NUMBER
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE del_Cliente 
( 
  paramClienteiD		IN Cliente.ClienteId%TYPE, 
  RSP					OUT NOCOPY VARCHAR2
) AS
  
  vValor  	NUMBER;
 
BEGIN
	RSP := 'KO';
	nombreProcedimento := 'del_Cliente'; 
	parametrosEntrada := 'Cliente: '|| paramClienteiD ;

-- comprobar los datos de entrada  
	IF (paramClienteiD  IS NULL) THEN
		MSG_NN := 'Falta Identificador del Cliente';
		RAISE ERR_NN;
	END IF;
	
	-- comprobar que  existe en la BD:   
	SELECT count(*) INTO existe
	FROM Cliente
	WHERE ClienteId = paramClienteId ;

	IF (existe = 0) THEN
		MSG_NN := 'El Cliente ' || paramClienteId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;
	
-- Obtenemos  ValorCarteraInversiones   
	SELECT ValorCarteraInversiones INTO  vValor
	FROM Cliente
	WHERE ClienteId = paramClienteId ;
	
-- comprobar que el valor de su cartera sea cero.   
	IF (vValor <> 0) THEN
	    MSG_NN := 'El Cliente ' || paramClienteId || 
					' no se puede dar de baja, porque tiene inversiones activas';
		RAISE ERR_NN;
	END IF;
	
	UPDATE Cliente SET
		EstaActivo = 'I'
	WHERE ClienteId = paramClienteId;
	
	RSP := 'OK';
	MSG_OK := 'Baja Cliente realizada correctamente.';
	
	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
	  
END del_Cliente ;

/* ****************************************************
PROCEDIMENTO 		Alta Cartera
@Nombre:			ins_Cartera
@Funcionalidad:		Alta Tabla Cartera.
@restricciones:		Ninguna.
@params IN:			paramClienteId NUMBER , 
					paramCarteraDescricion VARCHAR2(255)
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE ins_Cartera
(
  paramClienteId				IN Cartera.ClienteId%TYPE, 
  paramCarteraDescricion		IN Cartera.CarteraDescripcion%TYPE, 
  RSP							OUT NOCOPY VARCHAR2
) AS
  
BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_Cartera'; 
	parametrosEntrada := 'Cartera: ' || paramClienteId || ' - '|| paramCarteraDescricion ;

-- comprobar los datos de entrada  
	IF (paramClienteId = 0) THEN
		MSG_NN := 'Falta El Cliente';
		RAISE ERR_NN;
	END IF;
	IF (paramCarteraDescricion IS NULL) THEN
		MSG_NN := 'Falta descripción de la cartera';
		RAISE ERR_NN;	
	END IF;	
	
-- comprobar Cliente exista en la tabla Cliente  
	SELECT count(*) INTO existe
	FROM Cliente
	WHERE ClienteId = paramClienteId;

	IF (existe = 0) THEN
		MSG_NN := 'El Cliente ' || paramClienteId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;
	
-- comprobar Cliente no exista en la tabla cartera
-- por restricciones establecidas un cliente tan solo tiene una cartera.
	SELECT count(*) INTO existe
	FROM Cartera
	WHERE ClienteId = paramClienteId;

	IF (existe > 0) THEN
		MSG_NN := 'El Cliente ' || paramClienteId || ' ya tiene una Cartera de Inversiones.';
		RAISE ERR_NN;
	END IF;
	
-- INSERT nueva Cartera
	INSERT INTO Cartera (ClienteId, CarteraDescripcion,  EstaActivo)
  VALUES (paramClienteId, paramCarteraDescricion,'A');

	RSP:='OK';
	MSG_OK := 'Alta Cartera realizada correctamente.';
	
	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
	  
END ins_Cartera ;

/* ****************************************************
PROCEDIMENTO 		Modificación Cartera
@Nombre:			upd_Cartera
@Funcionalidad:		Modificación Tabla de Carteras.
@restricciones:		Que exita la Cartera.
@params IN:			paramCarteraiD NUMBER, 
					paramClienteId NUMBER , 
					paramCarteraDescricion VARCHAR2(255)
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE upd_Cartera 
(
  paramCarteraiD		    IN Cartera.CarteraId%TYPE, 
  paramClienteId		  	IN Cartera.ClienteId%TYPE, 
  paramCarteraDescricion	IN Cartera.CarteraDescripcion%TYPE, 
  RSP						OUT NOCOPY VARCHAR2
) AS

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'upd_Cartera'; 
	parametrosEntrada := 'Cartera: '|| paramCarteraiD || ' - '|| 
									   paramClienteId || ' - '|| 
									   paramCarteraDescricion  ;

-- comprobar los datos de entrada  
	IF (paramCarteraiD = 0) THEN
		MSG_NN := 'Falta Identificador de Cartera';
		RAISE ERR_NN;
	END IF;
	IF (paramClienteId = 0) THEN
		MSG_NN := 'Falta El Cliente';
		RAISE ERR_NN;
	END IF;
	IF (paramCarteraDescricion IS NULL) THEN
		MSG_NN := 'Falta descripción de la cartera';
		RAISE ERR_NN;	
	END IF;	
	
-- comprobar que  existe en la BD:   
	SELECT count(*) INTO existe
	FROM Cartera
	WHERE CarteraId = paramCarteraId AND
		ClienteId = paramClienteId;

	IF (existe = 0) THEN
		MSG_NN := 'La Cartera ' || paramCarteraId || ' Cliente ' || 
					paramClienteId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;

-- Modificación del Cartera
	UPDATE Cartera SET
		CarteraDescripcion = paramCarteraDescricion
	WHERE CarteraId = paramCarteraId;
	
	RSP:='OK';
	MSG_OK := 'Modificación Cartera realizada correctamente.';
	
	 -- Gravamos el LOG
  CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
	  
END upd_Cartera ;

/* ****************************************************
PROCEDIMENTO 		Baja Cartera
@Nombre:			del_Cartera
@Funcionalidad:		Inactivar la Cartera. 
					Se realiza una baja lógica, 
					modificando el estado a inactivo. 
@restricciones:		Que exita la Cartera.
					Que el valor cartera sea = 0
@params IN:			paramCarteraiD NUMBER,
					paramClienteId NUMBER
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE del_Cartera 
( 
  paramCarteraiD		IN Cartera.CarteraId%TYPE, 
  paramClienteId		IN Cartera.ClienteId%TYPE, 
  RSP					OUT NOCOPY VARCHAR2
) AS
  vValor  	NUMBER;
BEGIN
	RSP := 'KO';
	nombreProcedimento := 'del_Cartera'; 
	parametrosEntrada := 'Cartera: '|| paramCarteraiD || ' - '|| paramClienteId;

-- comprobar los datos de entrada  
	IF (paramCarteraiD = 0) THEN
		MSG_NN := 'Falta Identificador de Cartera';
		RAISE ERR_NN;
	END IF;
	
	IF (paramClienteId = 0) THEN
		MSG_NN := 'Falta Identificador de Cliente';
		RAISE ERR_NN;
	END IF;
	
-- comprobar que  existe en la BD:   
	SELECT count(*) INTO existe 
	FROM Cartera
	WHERE CarteraId = paramCarteraId AND
		ClienteId = paramClienteId;	

	IF (existe = 0) THEN
		MSG_NN := 'La Cartera ' || paramCarteraId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;
		
-- Baja lógica de Cartera
	UPDATE Cartera SET
		EstaActivo = 'I'
	WHERE CarteraId = paramCarteraId AND
		ClienteId = paramClienteId;
	
	RSP := 'OK';
	MSG_OK := 'Baja Cartera realizada correctamente.';
	
	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);
	
	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;
    
	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);
    
    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;
 	 
    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
	  
END del_Cartera ;

END TABLASMANTENIMIENTO;

/


