﻿--------------------------------------------------------
-- Archivo creado  - domingo-abril-19-2015   
--------------------------------------------------------
--------------------------------------------------------
--  DDL for Package GestionTest 
--------------------------------------------------------

create or replace PACKAGE BODY GESTIONTEST AS

  existe	NUMBER;
 
  parametrosEntrada		VARCHAR2(1024);
  nombreProcedimento    VARCHAR2(50);
  parametrosSalida      VARCHAR2(255);
   
  ERR_NUM	NUMBER(6);
  ERR_NN	EXCEPTION;

  MSG_ERR	VARCHAR2(96);
  MSG_NN 	VARCHAR2(96);
  MSG_OK 	VARCHAR2(96);

/* ****************************************************
PROCEDIMENTO 		Alta AccionHistorico
@Nombre:			ins_AccionHistorico
@Funcionalidad:		Alta Tabla AccionHistorico.
@restricciones:		Ninguna.
@params IN:			paramISINId CHAR(12) , 
					paramFechaCierre CHAR(9), 
					paramValorCierre NUMBER
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE ins_AccionHistorico
(
  paramISINId 			IN AccionHistorico.ISINId%TYPE,
  paramFechaCierre 		IN AccionHistorico.FechaCierre%TYPE,
  paramValorCierre		IN AccionHistorico.ValorCierre%TYPE,
  RSP					OUT NOCOPY VARCHAR2
) AS

  maximaFechaCierre	AccionHistorico.FechaCierre%TYPE;

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_AccionHistorico';
	parametrosEntrada := 'AccionHistorico: '|| paramISINId || ' - '|| 
											   paramFechaCierre || ' - '|| 
											   paramValorCierre ;

-- comprobar los datos de entrada
	IF (paramISINId  IS NULL) THEN
		MSG_NN := 'Falta Identificador de de la Acción';
		RAISE ERR_NN;
	END IF;
	IF (paramFechaCierre IS NULL) THEN
		MSG_NN := 'Falta la fecha de la Acción';
		RAISE ERR_NN;
	END IF;
	IF (paramValorCierre IS NULL) THEN
		MSG_NN := 'Falta el valor de la Acción';
		RAISE ERR_NN;
	END IF;

	IF NOT (Utilidades.Es_Fecha(TO_CHAR(paramFechaCierre))) THEN
		MSG_NN := 'La fecha introducida es erronea';
		RAISE ERR_NN;
	END IF;

	-- comprobar si el paramISINId existe en la BD:
	SELECT count(*) INTO existe
	FROM Accion
	WHERE ISINId = paramISINId;

	IF (existe = 0) THEN
		MSG_NN := 'El Acción ' || paramISINId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;
	
	SELECT MAX(FechaCierre) INTO maximaFechaCierre
	FROM AccionHistorico; 
	
	/* no podemos pasar cotizaciones anteriores a la última fecha introducida */
	IF (maximaFechaCierre > paramFechaCierre) THEN
		MSG_NN := 'La fecha de cierre ' || paramFechaCierre || ' es inferior a la última introducida ' || maximaFechaCierre ;
		RAISE ERR_NN;
	END IF;

-- INSERT del nuevo AccionHistorico
	INSERT INTO AccionHistorico (ISINId, FechaCierre, ValorCierre)
	VALUES (paramISINId, paramFechaCierre, paramValorCierre);

	RSP:='OK';
	MSG_OK := 'Alta AccionHistorico realizada correctamente.';

	 -- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);

	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;

	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

   DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;

    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);

END ins_AccionHistorico ;

/* ****************************************************
PROCEDIMENTO 		Alta OrdenInversion
@Nombre:			ins_OrdenInversion
@Funcionalidad:		Alta Tabla OrdenInversion.
@restricciones:		Ninguna.
@params IN:			paramISINId CHAR(12) , 
					paramTipoOrdenId CHAR(1), 
					paramTipoOrigenId CHAR(1),
					paramCantidad NUMBER, 
					paramPrecioMinimo NUMBER,
					paramCarteraId NUMBER, 
					paramClienteId NUMBER
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@params opcionales	paramFechaIntroduccion DATE
@error				MSG_ERR con la información del error
**************************************************** */
  PROCEDURE ins_OrdenInversion
(
  paramISINId				IN OrdenInversion.ISINId%TYPE,
  paramTipoOrdenId			IN OrdenInversion.TipoOrdenId%TYPE,
  paramTipoOrigenId			IN OrdenInversion.TipoOrigenId%TYPE,
  paramCantidad				IN OrdenInversion.Cantidad%TYPE,
  paramPrecioMinimo			IN OrdenInversion.PrecioMinimo%TYPE,
  paramCarteraId			IN OrdenInversion.CarteraId%TYPE,
  paramClienteId			IN OrdenInversion.ClienteId%TYPE,
  RSP						OUT NOCOPY VARCHAR2,
  paramFechaIntroduccion    IN OrdenInversion.FechaIntroduccion%type := to_date('01/01/00','DD/MM/YY')
  /* parámetros  opcionales, van siempre al final. 
	 La fecha como parámetro opcional, nos permitirá crear los datos necesarios 
	 para el entorno de pruebas. En el entorno real de explotación la fecha de 
	 introducción será la fecha del sistema. */
) AS

  vEstadoId         	OrdenEstado.EstadoId%TYPE;
  vFechaIntroduccion    OrdenInversion.FechaIntroduccion%type;
  vFechaEjecucion       OrdenInversion.FechaEjecucion%type;
  vCantidad		        OrdenInversion.Cantidad%type;
  vSignoTipoOrden		NUMBER;
  vEjecucionPrecio		NUMBER;

  oldCantidad		    OrdenInversion.Cantidad%type;
 
BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_OrdenInversion';
	parametrosEntrada := 'OrdenInversion: ' || paramISINId || ' - '|| 
											   paramTipoOrdenId || ' - '|| 
											   paramTipoOrigenId || ' - '||
											   paramCantidad || ' - '|| 
											   paramPrecioMinimo || ' - '||
											   paramCarteraId || ' - '|| 
											   paramClienteId;

-- comprobar los datos de entrada

	IF (paramISINId IS NULL) THEN
		MSG_NN := 'Falta el Identificador de la Acción';
		RAISE ERR_NN;
	END IF;
	IF (ParamTipoOrdenId IS NULL) THEN
		MSG_NN := 'Falta el Tipo Orden';
		RAISE ERR_NN;
	END IF;
	IF (paramTipoOrigenId IS NULL) THEN
		MSG_NN := 'Falta el Tipo Origen';
		RAISE ERR_NN;
	END IF;
	IF (paramCantidad = 0) THEN
		MSG_NN := 'Falta la cantidad';
		RAISE ERR_NN;
	ELSE
		IF (paramCantidad <=0) THEN
			MSG_NN := 'La cantidad debe ser mayor de cero.';
		RAISE ERR_NN;
		END IF;
	END IF;
	IF (paramPrecioMinimo != 0) THEN /* no es un campo obligatorio */
		IF (paramPrecioMinimo <0) THEN
			MSG_NN := 'La precio minimo no puede ser negativo.';
		RAISE ERR_NN;
		END IF;
	END IF;
	IF (paramCarteraId  = 0) THEN
		MSG_NN := 'Falta el identificador de la cartera';
		RAISE ERR_NN;
	END IF;
	IF (paramClienteId = 0) THEN
		MSG_NN := 'Falta el Cliente';
		RAISE ERR_NN;
	END IF;

-- comprobar Cartera / Cliente exista en la BD:
	SELECT count(*) INTO existe
	FROM Cartera
	WHERE CarteraId = paramCarteraId AND
		ClienteId = paramClienteId;

	IF (existe = 0) THEN
		MSG_NN := 'La Cartera ' || paramCarteraId || 
				' - Cliente ' || paramClienteId ||' no existe en la BD.';
		RAISE ERR_NN;
	END IF;

-- comprobar ISIN exista en la BD:
	SELECT count(*) INTO existe
	FROM Accion
	WHERE ISINId = paramISINId;

	IF (existe = 0) THEN
		MSG_NN := 'El codigo ISIN ' || paramISINId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;

-- comprobar Tipo Orden exista en la BD:
	SELECT count(*) INTO existe
	FROM TipoOrden
	WHERE TipoOrdenId = paramTipoOrdenId;

	IF (existe = 0) THEN
		MSG_NN := 'El identificador Tipo Orden ' || paramTipoOrdenId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;

-- comprobar Tipo Origen exista en la BD:
	SELECT count(*) INTO existe
	FROM TipoOrigen
	WHERE TipoOrigenId = paramTipoOrigenId;

	IF (existe = 0) THEN
		MSG_NN := 'El identificador Tipo Origen ' || paramTipoOrigenId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;

	vEstadoId:= 'P';
	vSignoTipoOrden := 0;
	vEjecucionPrecio := 0;

	IF (paramFechaIntroduccion = to_date('01/01/00','DD/MM/YY')) THEN
		vFechaIntroduccion := SYSDATE;
	ELSE
		vFechaIntroduccion := paramFechaIntroduccion;
	END IF;

	vFechaEjecucion := vFechaIntroduccion;

-- INSERT nueva OrdenInversion
	INSERT INTO OrdenInversion (ISINId, TipoOrdenId, TipoOrigenId, FechaIntroduccion, EstadoId,FechaEjecucion,
								Cantidad, PrecioMinimo, EjecucionPrecio, SignoTipoOrden, CarteraId, ClienteId)
	VALUES (paramISINId, paramTipoOrdenId, paramTipoOrigenId, vFechaIntroduccion, vEstadoId, vFechaEjecucion,
		paramCantidad, paramPrecioMinimo, vEjecucionPrecio, vSignoTipoOrden, paramCarteraId, paramClienteId);

	RSP:='OK';
	MSG_OK := 'Alta OrdenInversion realizada correctamente.';

	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);

	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

	-- Actualizamos datos estadísticos
	SELECT count(*) INTO existe
	FROM SituacionCartera
	WHERE  ClienteId = paramClienteId AND
			ISINId = paramISINId;
		
	IF (paramTipoOrdenId = 'V')	THEN
		vCantidad := paramCantidad ;
	ELSE
		vCantidad := 0 ;
	END IF;	
	
	IF (existe = 0) THEN
		INSERT INTO SituacionCartera (ClienteId, ISINId, ValorCotizacionUlt, FechaCotizacionUlt, Importe, Cantidad,CantidadPendienteVenta)
		VALUES (paramClienteId, paramISINId, 0, SYSDATE, 0, 0, vCantidad);
	ELSE
		SELECT 
		 CantidadPendienteVenta INTO  oldCantidad
		FROM SituacionCartera
		WHERE  ClienteId = paramClienteId AND
				ISINId = paramISINId;
				
		UPDATE SituacionCartera SET
		  CantidadPendienteVenta = oldCantidad + vCantidad
		WHERE  ClienteId = paramClienteId AND
				ISINId = paramISINId;
	END IF; 
		
	EXCEPTION

	WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;

	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

	WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;

    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);

END ins_OrdenInversion ;

/* ****************************************************
PROCEDIMENTO 		Modificación OrdenInversion
@Nombre:			upd_OrdenInversion
@Funcionalidad:		Modificación Tabla OrdenInversion.
@restricciones:		Una orden ejecutada/cancelada no se podra modificar.
					La ejecución de una orden de venta comprobara que tenga suficiente cantidad
					de acciones disponibles el cliente.
@params IN:			paramOrdenInversionId NUMBER,
					paramCarteraId NUMBER, 
					paramClienteId NUMBER,
					paramCantidad NUMBER, 
					paramPrecioMinimo NUMBER,
					paramEstadoId CHAR(1)					
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@params opcionales	paramFechaEjecucion DATE					
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE upd_OrdenInversion
(
  paramOrdenInversionId			IN OrdenInversion.OrdenInversionId%TYPE,
  paramCarteraId				IN OrdenInversion.CarteraId%TYPE,
  paramClienteId				IN OrdenInversion.ClienteId%TYPE,
  paramCantidad					IN OrdenInversion.Cantidad%TYPE,
  paramEjecucionPrecio			IN OrdenInversion.EjecucionPrecio%TYPE,
  paramEstadoId					IN OrdenInversion.EstadoId%TYPE,
  RSP							OUT NOCOPY VARCHAR2,
  paramFechaEjecucion           IN OrdenInversion.FechaEjecucion%type := to_date('01/01/00','DD/MM/YY') 
   /* parametros opcionales, van siempre al final. 
	 La fecha como parámetro opcional, nos permitirá crear los datos necesarios 
	 para el entorno de pruebas. En el entorno real de explotación la fecha de 
	 introducción será la fecha del sistema. */
) AS

  oldTipoOrdenId  				OrdenInversion.TipoOrdenId%TYPE;
  oldEstadoId     				OrdenInversion.EstadoId%TYPE;
  oldCantidad     				OrdenInversion.Cantidad%TYPE;
  oldEjecucionPrecio			OrdenInversion.EjecucionPrecio%TYPE;
  oldFechaEjecucion 			OrdenInversion.FechaEjecucion%TYPE;
  oldSignoTipoOrden 			OrdenInversion.SignoTipoOrden%TYPE;
  oldISINId       				OrdenInversion.ISINId%TYPE;
  vCantidad						OrdenInversion.Cantidad%TYPE;
  vCantidadEstadistica			OrdenInversion.Cantidad%TYPE;
  vCantidadDisponible			OrdenInversion.Cantidad%TYPE;
  vEstadoId						OrdenInversion.EstadoId%TYPE;
  vEjecucionPrecio				OrdenInversion.EjecucionPrecio%TYPE;
  vFechaEjecucion				DATE;
  vSignoTipoOrden 				OrdenInversion.SignoTipoOrden%TYPE;
  vSignoVenta     				OrdenInversion.SignoTipoOrden%TYPE;
  vNumeroClientesPoseedores 	NUMBER;

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'upd_OrdenInversion';
	parametrosEntrada := 'OrdenInversion: ' || paramOrdenInversionId || ' - '|| 
											   paramCarteraId || ' - '|| 
											   paramClienteId || ' - '||
											   paramCantidad || ' - '|| 
											   paramEjecucionPrecio || ' - '|| 
											   paramEstadoId ;

-- comprobar los datos de entrada
	IF (paramOrdenInversionId = 0) THEN
		MSG_NN := 'Falta el identificador de la Oden de Training';
		RAISE ERR_NN;
	END IF;

	IF (paramCarteraId  = 0) THEN
		MSG_NN := 'Falta el identificador de la cartera';
		RAISE ERR_NN;
	END IF;

	IF (paramClienteId  = 0) THEN
		MSG_NN := 'Falta el Cliente';
		RAISE ERR_NN;
	END IF;

-- comprobar que  existe en la BD:
	SELECT count(*), TipoOrdenId, EstadoId, Cantidad, EjecucionPrecio, FechaEjecucion, ISINId  INTO
        existe , oldTipoOrdenId, oldEstadoId, oldCantidad, oldEjecucionPrecio, oldFechaEjecucion, oldISINId
	FROM OrdenInversion
	WHERE OrdenInversionId = paramOrdenInversionId AND
		  CarteraId = paramCarteraId AND
		  ClienteId = paramClienteId
	GROUP BY TipoOrdenId, EstadoId, Cantidad, EjecucionPrecio, FechaEjecucion, ISINId ;

	IF (existe = 0) THEN
		MSG_NN := 'La Orden Inversion ' || paramOrdenInversionId ||
				  ' Cartera ' || paramCarteraId ||
				  ' Cliente ' || paramClienteId ||' no existe en la BD.';
		RAISE ERR_NN;
	END IF;

-- Solo podremos modificar ordenes en Estado pendiente.
	IF NOT (oldEstadoId = 'P') THEN
		MSG_NN := 'La Orden Inversion ' || paramOrdenInversionId ||
				  ' Cartera ' || paramCarteraId ||
				  ' Cliente ' || paramClienteId || ' se encuentra en Estado '
				  || oldEstadoId || ' y no se puede modificar ';
		RAISE ERR_NN;
	END IF;

	IF (paramCantidad IS NULL OR paramCantidad = 0) THEN
		vCantidad := oldCantidad;
	ELSE
		vCantidad := paramCantidad;
	END IF;
		
	IF (paramEstadoId IS NULL) THEN
		vEstadoId := oldEstadoId;
	ELSE
		vEstadoId := paramEstadoId;
	END IF;

	/* en el caso de que se trate de una modificación y el estado siga siendo
	'P' =  Pendiente debemos preservar la cantidad pendiente que utilizamos
	para estadisticas	
	Este problema no se presenta en el estadistico de las ordenes 'E' = Ejecutadas*/
	vCantidadEstadistica := vCantidad;
	IF (vEstadoId = 'P') THEN
		vCantidadEstadistica := vCantidad - paramCantidad;
	END IF;
	
	SELECT SignoTipoOrden INTO vSignoTipoOrden
	FROM TipoOrden
	WHERE TipoOrdenId = oldTipoOrdenId ;

	vFechaEjecucion := oldFechaEjecucion;
	vEjecucionPrecio := oldEjecucionPrecio;

	IF (vEstadoId = 'E')	THEN /*Orden de ejecución */
		IF (paramEjecucionPrecio IS NULL) THEN
			MSG_NN := 'Falta el Precio';
			RAISE ERR_NN;
		END IF;

		IF (paramEjecucionPrecio <=0) THEN
			MSG_NN := 'El precio ejecución debe ser mayor de cero.';
		RAISE ERR_NN;
		END IF;

		IF (paramFechaEjecucion = to_date('01/01/00','DD/MM/YY')) THEN
			vFechaEjecucion := SYSDATE;
		ELSE
			vFechaEjecucion := paramFechaEjecucion;
		END IF;

		vEjecucionPrecio := paramEjecucionPrecio;
	END IF;

/* El Tipo Orden nos proporciona el signo de la orden.
   En el análisis se determinan en estos momentos
	- Compras ----> Tendra signo = 1
	- Ventas  ----> Tendra signo = -1
   El diseño de la tabla con un campo signo nos deja la posibilidad de gestionar
   nuevos tipos con la misma filosofía. Por ejemplo podríamos tener Traspasos
   positivos, Traspasos negativos, Cesiones, etc..
   Por tanto deberemos controlar que la cantidad, disponible por el cliente, sea
   igual o inferior a la de la orden tan solo si el signo es negativo.
 */
	IF (vEstadoId = 'E') THEN
		IF (vSignoTipoOrden < 0) THEN 
		/* Cuando se trata de una venta comprobamos que tengamos suficiente cantidad */
			SELECT SUM(Cantidad * SignoTipoOrden) INTO vCantidadDisponible
			FROM OrdenInversion
			WHERE  CarteraId = paramCarteraId AND
				  ClienteId = paramClienteId;

			If (vCantidadDisponible < vCantidad) THEN
				MSG_NN := 'Cantidad disponible = ' || vCantidadDisponible || 
						  ' menor a Cantidad Orden =' || vCantidad;
				RAISE ERR_NN;
			END IF;

		END IF;
	ELSE
		/* Una orden Cancelada = Pendiente. No debe contar la cantidad. */
		vSignoTipoOrden := 0; 
	END IF;

-- Modificación del OrdenInversion
	UPDATE OrdenInversion SET
			EstadoId = vEstadoId ,
			Cantidad = vCantidad,
			SignoTipoOrden = vSignoTipoOrden,
			EjecucionPrecio = vEjecucionPrecio,
			FechaEjecucion = vFechaEjecucion
	WHERE OrdenInversionId = paramOrdenInversionId AND
		  CarteraId = paramCarteraId AND
		  ClienteId = paramClienteId;

	RSP:='OK';
	MSG_OK := 'Modificación OrdenInversion realizada correctamente.';

	-- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);

	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

	-- Actualizamos datos estadísticos
	IF (vEstadoId = 'E')	THEN /*Orden de ejecución */
		UPDATE SituacionCartera SET
			Cantidad = Cantidad + (vCantidad * vSignoTipoOrden)
		WHERE  ClienteId = paramClienteId AND
			ISINId = oldISINId;

		/* Actualizamos en  ACCION el número de Clientes que
		   la tiene activa en NumeroClientesPoseedores*/
		SELECT COUNT(DISTINCT ClienteId) INTO vNumeroClientesPoseedores
		FROM
			(SELECT  ClienteId, sum(Cantidad * SignoTipoOrden)
			 From OrdenInversion
			 WHERE ISINID = oldISINId
		  	 GROUP BY ClienteId
			 HAVING SUM(Cantidad * SignoTipoOrden) > 0);
		UPDATE Accion SET
			NumeroClientesPoseedores = vNumeroClientesPoseedores
		WHERE  ISINId = oldISINId;
	END IF;

 /*  Actualizamos la Ventas pendientes en la tabla SituacionCartera */
    IF (oldTipoOrdenId = 'V')	THEN /*Orden de Venta */
		IF (vEstadoId = 'P')	THEN /*Orden Pendiente */
		  vSignoVenta := 1; /* Las ordenes pendientes incrementan */
		ELSE
		  vSignoVenta := -1; /* Las ordenes ejecutadas o canceladas decrementan */
		END IF;
		UPDATE SituacionCartera SET
		  CantidadPendienteVenta = CantidadPendienteVenta + (vCantidadEstadistica * vSignoVenta)
		WHERE  ClienteId = paramClienteId AND
		  ISINID = oldISINId;
	END IF;
	
  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;

	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;

    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
END upd_OrdenInversion ;

/* ****************************************************
PROCEDIMENTO 		Alta OrdenTrading
@Nombre:			ins_OrdenTrading
@Funcionalidad:		Alta Tabla OrdenTrading.
@restricciones:		Ninguna.
@params IN:			paramCarteraId NUMBER, 
					paramClienteId NUMBER,
					paramISINId	CHAR(12),
					paramTipoOrdenId CHAR(1),
					paramCantidad NUMBER, 
					paramPrecioMinimo NUMBER,
					paramTradingId CHAR(10)					
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
  PROCEDURE ins_OrdenTrading
(
  paramCarteraId		IN OrdenTrading.CarteraId%TYPE,
  paramClienteId		IN OrdenTrading.ClienteId%TYPE,
  paramISINId			IN OrdenTrading.ISINId%TYPE,
  paramTipoOrdenId		IN OrdenTrading.TipoOrdenId%TYPE,
  paramCantidad			IN OrdenTrading.Cantidad%TYPE,
  paramPrecioMinimo		IN OrdenTrading.PrecioMinimo%TYPE,
  paramTradingId		IN OrdenTrading.TradingId%TYPE,
  RSP					OUT NOCOPY VARCHAR2
) AS

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_OrdenTrading';
	parametrosEntrada := 'OrdenTrading: ' || paramCarteraId || ' - '|| 
											 paramClienteId || ' - '|| 
										     paramISINId || ' - '||
											 paramTipoOrdenId || ' - '|| 
									         paramCantidad || ' - '|| 
											 paramPrecioMinimo || ' - '||
											 paramTradingId;

-- comprobar los datos de entrada
	IF (paramCarteraId  = 0) THEN
		MSG_NN := 'Falta el identificador de la cartera';
		RAISE ERR_NN;
	END IF;
	IF (paramClienteId  = 0) THEN
		MSG_NN := 'Falta el Cliente';
		RAISE ERR_NN;
	END IF;
	IF (paramISINId IS NULL) THEN
		MSG_NN := 'Falta el Identificador de la Acción';
		RAISE ERR_NN;
	END IF;
	IF (ParamTipoOrdenId IS NULL) THEN
		MSG_NN := 'Falta el el Tipo de Orden';
		RAISE ERR_NN;
	END IF;
	IF (paramCantidad = 0) THEN
		MSG_NN := 'Falta la cantidad';
		RAISE ERR_NN;
	END IF;
	IF (paramPrecioMinimo = 0) THEN
		MSG_NN := 'Falta el precio máximo';
		RAISE ERR_NN;
	END IF;
	IF (paramTradingId IS NULL) THEN
		MSG_NN := 'Falta el identificador Traiding';
		RAISE ERR_NN;
	END IF;

-- comprobar Cliente exista en la BD:
	SELECT count(*) INTO existe
	FROM Cliente
	WHERE ClienteId = paramClienteId;

	IF (existe = 0) THEN
		MSG_NN := 'El Cliente ' || paramClienteId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;
-- comprobar Tipo Orden exista en la BD:
	SELECT count(*) INTO existe
	FROM TipoOrden
	WHERE TipoOrdenId = paramTipoOrdenId;

	IF (existe = 0) THEN
		MSG_NN := 'El identificador Tipo Orden ' || paramTipoOrdenId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;
-- comprobar identificador Traiding exista en la BD:
	SELECT count(*) INTO existe
	FROM Trading
	WHERE TradingId = paramTradingId;

	IF (existe = 0) THEN
		MSG_NN := 'El identificador Traiding ' || paramTradingId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;

-- INSERT nueva OrdenTrading
	INSERT INTO OrdenTrading (CarteraId, ClienteId, ISINId, TipoOrdenId, 
							  Cantidad, PrecioMinimo, TradingId )
	VALUES (paramCarteraId, paramClienteId, paramISINId, paramTipoOrdenId,
			paramCantidad, paramPrecioMinimo, paramTradingId);

	RSP:='OK';
	MSG_OK := 'Alta OrdenTrading realizada correctamente.';

	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);

	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;

	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;

    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

END ins_OrdenTrading ;

/* ****************************************************
PROCEDIMENTO 		Modificación OrdenTrading
@Nombre:			upd_OrdenTrading
@Funcionalidad:		Modificación Tabla OrdenTrading.
@restricciones:		Ninguna.
@params IN:			paramOrdenTradingiD NUMBER,	
					paramCarteraId NUMBER, 
					paramClienteId NUMBER,
					paramISINId	CHAR(12),
					paramTipoOrdenId CHAR(1),
					paramCantidad NUMBER, 
					paramPrecioMinimo NUMBER,
					paramTradingId NUMBER					
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE upd_OrdenTrading
(
  paramOrdenTradingiD	IN OrdenTrading.OrdenTradingId%TYPE,
  paramCarteraId		IN OrdenTrading.CarteraId%TYPE,
  paramClienteId		IN OrdenTrading.ClienteId%TYPE,
  paramISINId			IN OrdenTrading.ISINId%TYPE,
  paramTipoOrdenId		IN OrdenTrading.TipoOrdenId%TYPE,
  paramCantidad			IN OrdenTrading.Cantidad%TYPE,
  paramPrecioMinimo		IN OrdenTrading.PrecioMinimo%TYPE,
  paramTradingId		IN OrdenTrading.TradingId%TYPE,
  RSP					OUT NOCOPY VARCHAR2
) AS

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'upd_OrdenTrading';
	parametrosEntrada := 'OrdenTrading: '|| paramOrdenTradingiD || ' - ' || 
											paramCarteraId || ' - '||
											paramClienteId || ' - ' || 
											paramISINId || ' - ' ||  
											paramTipoOrdenId || ' - ' || 
											paramCantidad || ' - ' || 
											paramPrecioMinimo || ' - ' || 
											paramTradingId ;

-- comprobar los datos de entrada
	IF (paramOrdenTradingId  IS NULL) THEN
		MSG_NN := 'Falta el identificador de la Oden de Training';
		RAISE ERR_NN;
	END IF;
	IF (paramCarteraId  IS NULL) THEN
		MSG_NN := 'Falta el identificador de la cartera';
		RAISE ERR_NN;
	END IF;
	IF (paramClienteId  IS NULL) THEN
		MSG_NN := 'Falta el Cliente';
		RAISE ERR_NN;
	END IF;
	IF (paramISINId IS NULL) THEN
		MSG_NN := 'Falta el Identificador de la Acción';
		RAISE ERR_NN;
	END IF;
	IF (ParamTipoOrdenId IS NULL) THEN
		MSG_NN := 'Falta el el Tipo de Orden';
		RAISE ERR_NN;
	END IF;
	IF (paramCantidad IS NULL) THEN
		MSG_NN := 'Falta la cantidad';
		RAISE ERR_NN;
	END IF;
	IF (paramPrecioMinimo IS NULL) THEN
		MSG_NN := 'Falta el precio máximo';
		RAISE ERR_NN;
	END IF;
	IF (paramTradingId IS NULL) THEN
		MSG_NN := 'Falta el identificador Traiding';
		RAISE ERR_NN;
	END IF;

-- comprobar que  existe en la BD:
	SELECT count(*) INTO existe
	FROM OrdenTrading
	WHERE OrdenTradingId = paramOrdenTradingId ;

	IF (existe = 0) THEN
		MSG_NN := 'El OrdenTrading ' || paramOrdenTradingId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;

-- Modificación del OrdenTrading
	UPDATE OrdenTrading SET
		CarteraId = paramCarteraId,
		ClienteId = paramClienteId,
		ISINId = paramISINId,
		TipoOrdenId = paramTipoOrdenId,
		Cantidad = paramCantidad,
		PrecioMinimo = paramPrecioMinimo,
		TradingId = paramTradingId
	WHERE OrdenTradingId = paramOrdenTradingId;

	RSP:='OK';
	MSG_OK := 'Modificación OrdenTrading realizada correctamente.';

	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);

	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;

	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;

    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);

END upd_OrdenTrading ;

/* ****************************************************
PROCEDIMENTO 		Baja OrdenTrading
@Nombre:			del_OrdenTrading
@Funcionalidad:		Baja Tabla OrdenTrading.
@restricciones:		Ninguna.
@params IN:			paramOrdenTradingiD NUMBER		
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE del_OrdenTrading
(
	paramOrdenTradingiD		    IN OrdenTrading.OrdenTradingId%TYPE,
	RSP							OUT NOCOPY VARCHAR2
) AS

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'del_OrdenTrading';
	parametrosEntrada := 'OrdenTrading: '|| paramOrdenTradingiD ;

-- comprobar los datos de entrada
	IF (paramOrdenTradingiD  IS NULL) THEN
		MSG_NN := 'Falta Identificador de OrdenTrading';
		RAISE ERR_NN;
	END IF;

-- comprobar que  existe en la BD:
	SELECT count(*) INTO existe
	FROM OrdenTrading
	WHERE OrdenTradingId = paramOrdenTradingId ;

	IF (existe = 0) THEN
		MSG_NN := 'La OrdenTrading ' || paramOrdenTradingId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;

	DELETE FROM OrdenTrading WHERE OrdenTradingId = paramOrdenTradingId;
	RSP := 'OK';
	MSG_OK := 'Baja OrdenTrading realizada correctamente.';

	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);

	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;

	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;

    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
END del_OrdenTrading ;

/* ****************************************************
PROCEDIMENTO 		Ejecución  OrdenTrading
@Nombre:			exe_OrdenTrading
@Funcionalidad:		Ejecución  Tabla OrdenTrading.
@restricciones:		La ejecución de una Orden Autmatica de Trading	automático, 
					se traducira en una inserción en la tabla OrdenInversion 
					(una inserción siempre la generamos en situación pendiente),
					después se producirá una modificación automática en la que
					la OrdeInversion cambiará su situación a ejecutada(en este punto 
					si es una orden de venta se comprobara que se tenga suficiente cantidad).
					Si no hay errores, se procederá a dar de baja el registro de OrdenTrading,
					ya que su información es irrelevante una vez ha cumplido su función.
@params IN:			paramOrdenTradingiD NUMBER		
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE exe_OrdenTrading
(
	paramOrdenTradingiD	    IN OrdenTrading.OrdenTradingId%TYPE,
	RSP						OUT NOCOPY VARCHAR2
) AS
	vOrdenInversionId 	NUMBER;
	vCarteraId			OrdenTrading.CarteraId%TYPE;
	vClienteId			OrdenTrading.ClienteId%TYPE;
	vISINId				OrdenTrading.ISINId%TYPE;
	vTipoOrdenId		OrdenTrading.TipoOrdenId%TYPE;
	vCantidad			OrdenTrading.Cantidad%TYPE;
	vPrecioMinimo		OrdenTrading.PrecioMinimo%TYPE;
BEGIN
	RSP := 'KO';
	nombreProcedimento := 'del_OrdenTrading';
	parametrosEntrada := 'OrdenTrading: '|| paramOrdenTradingiD ;

-- comprobar los datos de entrada
	IF (paramOrdenTradingiD = 0) THEN
		MSG_NN := 'Falta Identificador de OrdenTrading';
		RAISE ERR_NN;
	END IF;

-- comprobar que  existe en la BD:
	SELECT count(*), ISINId, TipoOrdenId, Cantidad, PrecioMinimo, CarteraId, ClienteId
	INTO existe, vISINId, vTipoOrdenId, vCantidad, vPrecioMinimo, vCarteraId, vClienteId
	FROM OrdenTrading
	WHERE OrdenTradingId = paramOrdenTradingId
	GROUP BY ISINId, TipoOrdenId, Cantidad, PrecioMinimo, CarteraId, ClienteId  ;

	SAVEPOINT noEncontrado; /* Sirve para marca un punto de referencia en la 
							  transacción para hacer un ROLLBACK parcial. */
							  
	-- Se ejecutara una orden de inserción en la tabla OrdenInversion
	-- (una inserción siempre la generamos en situación pendiente)
	ins_OrdenInversion(vISINId, vTipoOrdenId, 'A', vCantidad, vPrecioMinimo, vCarteraId, vClienteId, RSP);
	IF RSP = 'KO' THEN
	   MSG_NN := 'La OrdenTrading ' || paramOrdenTradingId || ' no se ha podido insertar en OrdenInversion.';
		RAISE ERR_NN;
	END IF;

	--Recuperamos el numero asignado a la nueva OrdenInversion.
	SELECT OrdenInversionId  INTO  vOrdenInversionId
	FROM OrdenInversion
	WHERE ISINId = vISINId AND
		  TipoOrdenId = vTipoOrdenId AND
		  ClienteId = vClienteId AND
		  CarteraId = vCarteraId AND
		  TipoOrigenId = 'A';

	-- Se ejecutara la modificación  a la OrdeInversion para cambiar la situación a ejecutada
	-- (en este punto si es una orden de venta se comprobara que se tenga suficiente cantidad)
	upd_OrdenInversion(vOrdenInversionId, vCarteraId, vClienteId, vCantidad, vPrecioMinimo, 'E', RSP);

	IF RSP = 'KO' THEN
		MSG_NN := 'La OrdenTrading ' || paramOrdenTradingId || ' no se ha podido modificar en OrdenInversion.';
		RAISE ERR_NN;
	END IF;

	-- Todo ha ido correcto. Debo restaurar el valor de RSP
	RSP := 'KO';

	-- Si no hay errores, se procederá¡ a dar de baja el registro de OrdenTrading,
	-- ya que es irrelevante su información una vez ha cumplido su función.
	IF (existe = 0) THEN
		MSG_NN := 'La OrdenTrading ' || paramOrdenTradingId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;

	DELETE FROM OrdenTrading WHERE OrdenTradingId = paramOrdenTradingId;
	RSP := 'OK';
	MSG_OK := 'Baja OrdenTrading realizada correctamente.';

	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);

	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);
	COMMIT;

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;

	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);
	-- Si no podemos Updatar deshacemos la Inserción.
	ROLLBACK TO noEncontrado;
  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;

    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);
 	-- Si no podemos Updatar deshacemos la Inserción.
	ROLLBACK TO noEncontrado;

END exe_OrdenTrading ;

/* ****************************************************
PROCEDIMENTO 		Alta ClienteHistorico
@Nombre:			ins_ClienteHistorico
@Funcionalidad:		Alta Tabla ClienteHistorico.
@restricciones:		Ninguna.
@params IN:			paramClienteId NUMBER,
					paramMes NUMBER,
					paramANO NUMBER,
					paramImporteInicial NUMBER,
					paramImporteFinal NUMBER
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE ins_ClienteHistorico
(
  paramClienteiD		    IN ClienteHistorico.ClienteId%TYPE,
  paramMes					IN ClienteHistorico.Mes%TYPE,
  paramANO					IN ClienteHistorico.ANO%TYPE,
  paramImporteInicial		IN ClienteHistorico.ImporteInicial%TYPE,
  paramImporteFinal			IN ClienteHistorico.ImporteFinal%TYPE,
  RSP						OUT NOCOPY VARCHAR2
) AS

BEGIN
	RSP := 'KO';
	nombreProcedimento := 'ins_ClienteHistorico';
	parametrosEntrada := 'ClienteHistorico: '|| paramClienteiD || ' - '|| 
												paramMes || ' - '|| 
												paramANO || ' - '||
										        paramImporteInicial || ' - '|| 
												paramImporteFinal ;

-- comprobar los datos de entrada
	IF (paramClienteiD = 0) THEN
		MSG_NN := 'Falta Identificador del Cliente';
		RAISE ERR_NN;
	END IF;
	IF (paramANO = 0) THEN
		MSG_NN := 'Falta el ANO';
		RAISE ERR_NN;
	END IF;
	IF (paramMes = 0) THEN
		MSG_NN := 'Falta el Mes';
		RAISE ERR_NN;
	END IF;
	
-- comprobar si el Cliente existe en la BD:
	SELECT count(*) INTO existe
	FROM Cliente
	WHERE ClienteId = paramClienteId;

	IF (existe = 0) THEN
		MSG_NN := 'El Cliente ' || paramClienteId || ' no existe en la BD.';
		RAISE ERR_NN;
	END IF;

-- INSERT nuevo ClienteHistorico
	INSERT INTO ClienteHistorico (ClienteId, ANO, Mes, ImporteInicial, 
								 ImporteFinal, Beneficio, Porcentaje)
	VALUES (paramClienteiD, paramANO, paramMes, paramImporteInicial, 
			paramImporteFinal, 0, 0 );

	RSP:='OK';
	MSG_OK := 'Alta ClienteHistorico realizada correctamente.';

	 -- Gravamos el LOG
  CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);

	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

  EXCEPTION

  WHEN ERR_NN THEN
    MSG_ERR := MSG_NN;

	-- Gravamos el LOG
    CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

    DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

  WHEN OTHERS THEN
      ERR_NUM := SQLCODE;
      MSG_ERR := SQLERRM;

    -- Gravamos el LOG
      CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

      DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);

END ins_ClienteHistorico ;

/* ****************************************************
PROCEDIMENTO 		Modificación ClienteHistorico
@Nombre:			upd_ClienteHistorico
@Funcionalidad:		Modificación Tabla ClienteHistorico.
@restricciones:		Ninguna.
@params IN:			paramClienteId NUMBER,
					paramMes NUMBER,
					paramANO NUMBER,
					paramImporteInicial NUMBER,
					paramImporteFinal NUMBER
@params OUT:		parametrosEntrada VARCHAR2(1024),
					nombreProcedimento VARCHAR2(50),
					parametrosSalida VARCHAR2(255),
					RSP
@error				MSG_ERR con la información del error
**************************************************** */
PROCEDURE upd_ClienteHistorico
(
  paramClienteiD	    IN ClienteHistorico.ClienteId%TYPE,
  paramMes				IN ClienteHistorico.Mes%TYPE,
  paramANO				IN ClienteHistorico.ANO%TYPE,
  paramImporteInicial	IN ClienteHistorico.ImporteInicial%TYPE,
  paramImporteFinal		IN ClienteHistorico.ImporteFinal%TYPE,
  RSP					OUT NOCOPY VARCHAR2
) AS

  vBeneficio			NUMBER;
  vPorcentaje			NUMBER;

 BEGIN
	RSP := 'KO';
	nombreProcedimento := 'upd_ClienteHistorico';
	parametrosEntrada := 'ClienteHistorico: '|| paramClienteiD || ' - '|| 
												paramMes|| ' - '|| 
												paramANO || ' - '||
										        paramImporteInicial || ' - '|| 
												paramImporteFinal ;

-- comprobar los datos de entrada
	IF (paramClienteiD = 0) THEN
		MSG_NN := 'Falta Identificador del Cliente';
		RAISE ERR_NN;
	END IF;
	IF (paramANO = 0) THEN
		MSG_NN := 'Falta el ANO';
		RAISE ERR_NN;
	END IF;
	IF (paramMes = 0) THEN
		MSG_NN := 'Falta el Mes';
		RAISE ERR_NN;
	END IF;
	
-- comprobar que  existe en la BD:
	SELECT count(*) INTO existe
	FROM ClienteHistorico
	WHERE ClienteId = paramClienteId AND
		ANO = paramANO AND
		Mes = paramMes;

	IF (existe = 0) THEN
		MSG_NN := 'El Cliente ' || paramClienteId || ' con Mes - ANO' ||
				paramMes || ' - ' || paramANO || ' no existe en la BD.';
	RAISE ERR_NN;
  END IF;

	vBeneficio := paramImporteFinal - paramImporteInicial;
	IF (paramImporteInicial = 0) THEN
		vPorcentaje := 100;
	ELSE
		vPorcentaje := (vBeneficio/paramImporteInicial)*100;
	END IF;

-- Modificación del ClienteHistorico
	UPDATE ClienteHistorico SET
		ImporteInicial = paramImporteInicial,
		ImporteFinal = paramImporteFinal,
		Beneficio = vBeneficio,
		Porcentaje = vPorcentaje
	WHERE ClienteId = paramClienteId AND
		ANO = paramANO AND
		Mes = paramMes;

	RSP:='OK';
	MSG_OK := 'Modificación ClienteHistorico realizada correctamente.';

	 -- Gravamos el LOG
	CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada, MSG_OK);

	DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_OK);

	EXCEPTION

	WHEN ERR_NN THEN
		MSG_ERR := MSG_NN;

		-- Gravamos el LOG
		CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

		DBMS_OUTPUT.PUT_LINE(RSP || ' - ' || nombreProcedimento || ' - ' || ParametrosEntrada|| ' - ' || MSG_ERR);

	WHEN OTHERS THEN
		  ERR_NUM := SQLCODE;
		  MSG_ERR := SQLERRM;

		-- Gravamos el LOG
		  CONTROL.GRAVALOG(RSP, nombreProcedimento, ParametrosEntrada,  MSG_ERR);

		  DBMS_OUTPUT.PUT_LINE('Error: ' || TO_CHAR(ERR_NUM) || ', ' || MSG_ERR);

END upd_ClienteHistorico ;

END GESTIONTEST;

/


