--------------------------------------------------------
-- Archivo creado  - miércoles-abril-29-2015   
--------------------------------------------------------
--------------------------------------------------------
--  DDL FOR Package Body CONSULTAS
--------------------------------------------------------

create or replace PACKAGE BODY CONSULTAS AS

/* a. Donada una data concreta i un identIFicador de client com a
paràmetres d’entrada: el llIStat amb la composició de la
cartera d’accions del client (vigent en la data indicada),
incloent-hi el valor en euros de les accions que tingui en
aquell moment. */
PROCEDURE Cons_1_Cartera_Cliente
(
  paramClienteId   IN Cliente.ClienteId%TYPE,
  paramFechaCierre IN AccionHIStorico.FechaCierre%TYPE
) AS
 v_nombre VARCHAR2(150);

	-- Cursor que retorna la cartera de un cliente a una fecha  según los parametros introducidos.
	CURSOR c_cartera IS
	SELECT 	OrdenInversion.ISINId, Ticker, TickerDescripcion,
			SUM(Cantidad * SignoTipoOrden) AS Volumen,
			SUM(Cantidad * SignoTipoOrden * ValorCierre) AS Valor
	FROM OrdenInversion
	LEFT JOIN AccionHIStorico ON
			OrdenInversion.ISINId = AccionHIStorico.ISINId
    LEFT JOIN Accion ON
			OrdenInversion.ISINId = Accion.ISINId
	WHERE OrdenInversion.ClienteId = paramClienteId AND
			AccionHIStorico.FechaCierre = paramFechaCierre AND
			OrdenInversion.FechaEjecucion <= paramFechaCierre
	GROUP BY OrdenInversion.ISINId, Ticker, TickerDescripcion
	ORDER BY Ticker;

	r_cartera c_cartera%ROWTYPE;

	BEGIN

    SELECT TO_CHAR( ClienteId) || ' - ' || Nombre INTO v_nombre
    FROM Cliente
    WHERE Clienteid = paramClienteId;

	DBMS_OUTPUT.ENABLE;
	
	DBMS_OUTPUT.PUT_LINE(CHR(10));/* CHR(10) = salto de línea (LF, line feed) */
	DBMS_OUTPUT.PUT_LINE('a. Donada una data concreta i un identificador de client com a');
	DBMS_OUTPUT.PUT_LINE('paràmetres d’entrada: el llistat amb la composició de la');
	DBMS_OUTPUT.PUT_LINE('cartera d’accions del client (vigent en la data indicada),');
	DBMS_OUTPUT.PUT_LINE('incloent-hi el valor en euros de les accions que tingui en');
	DBMS_OUTPUT.PUT_LINE('aquell moment. ');
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	DBMS_OUTPUT.PUT_LINE('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    DBMS_OUTPUT.PUT_LINE('COMPOSICIÓN DE LA CARTERA DE: '||v_nombre|| ' A FECHA: '||paramFechaCierre);
    DBMS_OUTPUT.PUT_LINE('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
   
	-- Se abre el cursor con los datos de la cartera
	OPEN c_cartera;
	FETCH c_cartera INTO r_cartera;
    IF c_cartera%FOUND THEN
        CLOSE c_cartera;
        FOR i IN c_cartera LOOP
          -- Si el cursor retorna regIStros es s'escriuen al buffer de l'output.
			DBMS_OUTPUT.PUT_LINE('Ticker: '||i.Ticker);
			DBMS_OUTPUT.PUT_LINE('Nombre: '||i.TickerDescripcion);
			DBMS_OUTPUT.PUT_LINE('ISIN: '||i.ISINId);
			DBMS_OUTPUT.PUT_LINE('Cantidad: '||i.Volumen);
			DBMS_OUTPUT.PUT_LINE('Valor: '|| ROUND(i.Valor,2));
			DBMS_OUTPUT.PUT_LINE(CHR(10));
			DBMS_OUTPUT.PUT_LINE('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
		END LOOP;
    ELSE
			DBMS_OUTPUT.PUT_LINE('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
			DBMS_OUTPUT.PUT_LINE('LA CONSULTA NO HA DEVUELTO RESULTADOS DE LA CARTERA.');
			DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
	END IF;
END Cons_1_Cartera_Cliente;

/* b. Donat un any concret com a paràmetre d’entrada: el llIStat
	  de tots els clients que hagin realitzat operacions durant
	  aquell any, incloent-hi per a cada client:
		-IdentIFicador del client.
		-Nom del client.
		-Nombre d’ordres de compra carregades durant l’any.
		-Nombre d’ordres de venda carregades durant l’any.
		-Nombre d’ordres de compra executades durant l’any.
		-Sumatori en euros de totes les ordres de compra executades durant l’any.
		-Valor mig en euros de cada operació de compra executada pel client durant l’any.
		-Nombre d’ordres de venda executades durant l’any.
		-Sumatori en euros de totes les ordres de venda executades durant l’any.
		-Valor mig en euros de cada operació de venda executada pel client durant l’any.
		-Nombre d’ordres de compra cancel·lades durant l’any.
		-Nombre d’ordres de venda cancel·lades durant l’any. */
PROCEDURE Cons_2_1_Clientes_Anual
(
  paramYear						IN NUMBER,
  paramClienteId 				IN OUT Cliente.ClienteId%TYPE,
  paramNombre					OUT Cliente.Nombre%TYPE,
  paramTotalComprasCargadas		OUT NUMBER,
  paramTotalVentasCargadas		OUT NUMBER,
  paramTotalComprasEjecutadas	OUT NUMBER,
  paramTotalVentasEjecutadas	OUT NUMBER,
  paramImporteComprasEjec		OUT NUMBER,
  paramValorMedioComprasEjec	OUT NUMBER,
  paramImporteVentasEjec		OUT NUMBER,
  paramValorMedioVentasEjec	  	OUT NUMBER,
  paramTotalComprasCanceladas	OUT NUMBER,
  paramTotalVentasCanceladas	OUT NUMBER
) AS

BEGIN

 SELECT OrdenInversion.ClienteId,  Cliente.Nombre,
		 (SELECT COUNT(*)
			FROM OrdenInversion
			WHERE TipoOrdenId = 'C' AND OrdenInversion.ClienteId = paramClienteId ) AS TotalComprasCargadas,
		 (SELECT COUNT(*)
			FROM OrdenInversion
			WHERE TipoOrdenId = 'V' AND OrdenInversion.ClienteId = paramClienteId) AS TotalVentasCargadas,
		 (SELECT COUNT(*)
			FROM OrdenInversion
			WHERE TipoOrdenId = 'C' AND 
			EstadoId = 'E' AND OrdenInversion.ClienteId = paramClienteId) AS TotalComprasEjecutadas,
		 (SELECT COUNT(*)
			FROM OrdenInversion
			WHERE TipoOrdenId = 'V' AND 
			EstadoId = 'E' AND OrdenInversion.ClienteId = paramClienteId) AS TotalVentasEjecutadas,	
		  (SELECT CASE COUNT(*)
				  WHEN 0 THEN 0
				  ELSE sum(cantidad * EjecucionPrecio)
          END
		 	FROM OrdenInversion
			WHERE TipoOrdenId = 'C' AND
				  EstadoId = 'E' AND OrdenInversion.ClienteId = paramClienteId) AS ImporteComprasEjecutadas,
		 (SELECT CASE COUNT(*)
				  WHEN 0 THEN 0
				  ELSE  sum(cantidad * EjecucionPrecio)	/ COUNT(*)
          END
			FROM OrdenInversion
			WHERE TipoOrdenId = 'C' AND
				  EstadoId = 'E' AND OrdenInversion.ClienteId = paramClienteId) AS ValorMedioComprasEjecutadas,
		 (SELECT CASE COUNT(*)
				  WHEN 0 THEN 0
				  ELSE sum(cantidad * EjecucionPrecio)
          END
		 	FROM OrdenInversion
			WHERE TipoOrdenId = 'V' AND
				  EstadoId = 'E' AND OrdenInversion.ClienteId = paramClienteId) AS ImporteVentasEjecutadas,
     (SELECT CASE COUNT(*)
				  WHEN 0 THEN 0
				  ELSE sum(cantidad * EjecucionPrecio)	/ COUNT(*)
          END
			FROM OrdenInversion
			WHERE TipoOrdenId = 'V' AND
				  EstadoId = 'E' AND OrdenInversion.ClienteId = paramClienteId) AS ValorMedioVentasEjecutadas,
		 (SELECT COUNT(*)
			FROM OrdenInversion
			WHERE TipoOrdenId = 'C'AND
				  EstadoId = 'C' AND OrdenInversion.ClienteId = paramClienteId) AS TotalComprasCanceladas,
		 (SELECT COUNT(*)
			FROM OrdenInversion
			WHERE TipoOrdenId = 'V' AND
				  EstadoId = 'C' AND OrdenInversion.ClienteId = paramClienteId) AS TotalVentasCanceladas
 INTO   paramClienteId, paramNombre, paramTotalComprasCargadas, paramTotalVentasCargadas , paramTotalComprasEjecutadas, 
		paramTotalVentasEjecutadas,  paramImporteComprasEjec, paramValorMedioComprasEjec, paramImporteVentasEjec, 
		paramValorMedioVentasEjec,   paramTotalComprasCanceladas, paramTotalVentasCanceladas
 FROM OrdenInversion
 LEFT JOIN Cliente ON
			OrdenInversion.ClienteId = Cliente.ClienteId
 WHERE EXTRACT(YEAR FROM FechaIntroduccion) = paramYear AND
       OrdenInversion.ClienteId = paramClienteId
 Group By  OrdenInversion.ClienteId,  Cliente.Nombre;
END Cons_2_1_Clientes_Anual;

/* b. Donat un any concret com a paràmetre d’entrada: el llIStat
	  de tots els clients que hagin realitzat operacions durant
	  aquell any, incloent-hi per a cada client:
		-IdentIFicador del client.
		-Nom del client.
		-Nombre d’ordres de compra carregades durant l’any.
		-Nombre d’ordres de venda carregades durant l’any.
		-Nombre d’ordres de compra executades durant l’any.
		-Sumatori en euros de totes les ordres de compra executades durant l’any.
		-Valor mig en euros de cada operació de compra executada pel client durant l’any.
		-Nombre d’ordres de venda executades durant l’any.
		-Sumatori en euros de totes les ordres de venda executades durant l’any.
		-Valor mig en euros de cada operació de venda executada pel client durant l’any.
		-Nombre d’ordres de compra cancel·lades durant l’any.
		-Nombre d’ordres de venda cancel·lades durant l’any. */  
PROCEDURE Cons_2_0_Clientes_Anual
(
  paramYear						IN NUMBER
 )
AS
 
  vNombre						 Cliente.Nombre%TYPE;
  vTotalComprasCargadas			 NUMBER;
  vTotalVentasCargadas			 NUMBER;
  vTotalComprasEjecutadas		 NUMBER;
  vTotalVentasEjecutadas		 NUMBER;
  vImporteComprasEjec		     NUMBER;
  vValorMedioComprasEjec	  	 NUMBER;
  vImporteVentasEjec		     NUMBER;
  vValorMedioVentasEjec	    	 NUMBER;
  vTotalComprasCanceladas		 NUMBER;
  vTotalVentasCanceladas		 NUMBER;
	-- Cursor que retorna los diferentes clientes en OrdenesInversion de un año.
	CURSOR c_cliente IS
	SELECT 	dIStinct(ClienteId) AS ClienteId
	FROM OrdenInversion
	WHERE EXTRACT(YEAR FROM FechaIntroduccion) = paramYear
	ORDER BY ClienteId;

	r_cliente c_cliente%ROWTYPE;

	BEGIN

	DBMS_OUTPUT.ENABLE;
	
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	DBMS_OUTPUT.PUT_LINE(' b. Donat un any concret com a paràmetre d’entrada: el llistat');
	DBMS_OUTPUT.PUT_LINE('de tots els clients que hagin realitzat operacions durant');
	DBMS_OUTPUT.PUT_LINE('aquell any, incloent-hi per a cada client:');
	DBMS_OUTPUT.PUT_LINE('-Identificador del client.');
	DBMS_OUTPUT.PUT_LINE('-Nom del client.');
	DBMS_OUTPUT.PUT_LINE('-Nombre d’ordres de compra carregades durant l’any.');
	DBMS_OUTPUT.PUT_LINE('-Nombre d’ordres de venda carregades durant l’any.');
	DBMS_OUTPUT.PUT_LINE('-Nombre d’ordres de compra executades durant l’any.');
	DBMS_OUTPUT.PUT_LINE('-Sumatori en euros de totes les ordres de compra executades durant l’any.');
	DBMS_OUTPUT.PUT_LINE('-Valor mig en euros de cada operació de compra executada pel client durant l’any.');
	DBMS_OUTPUT.PUT_LINE('-Nombre d’ordres de venda executades durant l’any.');
	DBMS_OUTPUT.PUT_LINE('-Sumatori en euros de totes les ordres de venda executades durant l’any.');
	DBMS_OUTPUT.PUT_LINE('-Valor mig en euros de cada operació de venda executada pel client durant l’any.');
	DBMS_OUTPUT.PUT_LINE('-Nombre d’ordres de compra cancel·lades durant l’any.');
	DBMS_OUTPUT.PUT_LINE('-Nombre d’ordres de venda cancel·lades durant l’any.');
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    DBMS_OUTPUT.PUT_LINE('CLIENTES EN CARTERA DEL AÑO: '||paramYear);
    DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    
	-- Se abre el cursor de clientes
	OPEN c_cliente;
	FETCH c_cliente INTO r_cliente;
    IF c_cliente%FOUND THEN
        CLOSE c_cliente;
        FOR i in c_cliente LOOP
	Cons_2_1_Clientes_Anual(paramYear , i.ClienteId, vNombre, vTotalComprasCargadas, vTotalVentasCargadas , 
							vTotalComprasEjecutadas, vTotalVentasEjecutadas, vImporteComprasEjec, 
							vValorMedioComprasEjec, vImporteVentasEjec, vValorMedioVentasEjec,
							vTotalComprasCanceladas, vTotalVentasCanceladas);

          -- Si el cursor retorna regIStros se escriben en el buffer del output.
			DBMS_OUTPUT.PUT_LINE('Cliente ........................: '|| i.ClienteId || ' - ' || vNombre);
			DBMS_OUTPUT.PUT_LINE('Compras Cargadas ...............: '|| vTotalComprasCargadas);
			DBMS_OUTPUT.PUT_LINE('Ventas Cargadas ................: '|| vTotalVentasCargadas);
			DBMS_OUTPUT.PUT_LINE('Compras Ejecutadas .............: '|| vTotalComprasEjecutadas);
			DBMS_OUTPUT.PUT_LINE('Valor € Compras Ejecutadas .....: '|| ROUND(vImporteComprasEjec,2));
			DBMS_OUTPUT.PUT_LINE('Valor Medio Compras Ejecutadas .: '|| ROUND(vValorMedioComprasEjec,2));
			DBMS_OUTPUT.PUT_LINE('Ventas Ejecutadas ..............: '|| vTotalVentasEjecutadas);
			DBMS_OUTPUT.PUT_LINE('Valor € Ventas Ejecutadas ......: '|| ROUND(vImporteVentasEjec,2));
			DBMS_OUTPUT.PUT_LINE('Valor Medio Ventas Ejecutadas ..: '|| ROUND(vValorMedioVentasEjec,2));
			DBMS_OUTPUT.PUT_LINE('Compras Canceladas .............: '|| vTotalComprasCanceladas);
			DBMS_OUTPUT.PUT_LINE('Ventas Canceladas ..............: '|| vTotalVentasCanceladas);
			DBMS_OUTPUT.PUT_LINE(CHR(10));
			DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
	    END LOOP;
    ELSE
			DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
			DBMS_OUTPUT.PUT_LINE('LA CONSULTA NO HA DEVUELTO RESULTADOS EN LA CARTERA DE ESTE AÑO.');
			DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
	END IF;
END Cons_2_0_Clientes_Anual;

/* c. Donat un any concret com a paràmetre d’entrada: el llistat
	  de totes les accions sobre les quals s’hagi realitzat alguna
	  operació, incloent-hi per a cada acció:
		-IdentIFicador de l’acció.
		-Nom complet de l’acció.
		-Nombre d’operacions de compra executades durant l’any.
		-Suma total en euros de les ordres de compra executades durant l’any sobre l’acció.
		-Valor mitjà en euros de les ordres de compra executades durant l’any sobre l’acció.
		-Nombre d’operacions de venda executades durant l’any sobre l’acció.
		-Suma total en euros de les ordres de venda executades sobre l’acció.
		-Revalorització de l’acció en percentatge des d’1 de gener de l’any indicat. */  
PROCEDURE Cons_3_1_Acciones_Anual
(paramYear						IN NUMBER,
  paramISINId 					IN OUT Accion.ISINId%TYPE,
  paramTicker					OUT Accion.Ticker%TYPE,
  paramTickerDescripcion		OUT Accion.TickerDescripcion%TYPE,
  paramTotalComprasEjec			OUT NUMBER,
  paramValorComprasEjec			OUT NUMBER,
  paramValorMedioComprasEjec	OUT NUMBER,
  paramTotalVentasEjec			OUT NUMBER,
  paramValorVentasEjec			OUT NUMBER
) AS

BEGIN
 SELECT OrdenInversion.ISINId,  Accion.Ticker, Accion.TickerDescripcion,
		 (SELECT COUNT(*)
			FROM OrdenInversion
			WHERE TipoOrdenId = 'C' AND
				  EstadoId = 'E'AND
				  OrdenInversion.ISINId = paramISINId ) AS TotalComprasEjec	,
		 (SELECT CASE COUNT(*)
				  WHEN 0 THEN 0
				  ELSE sum(cantidad * EjecucionPrecio)
          END
		 	FROM OrdenInversion
			WHERE TipoOrdenId = 'C' AND
				  EstadoId = 'E' AND OrdenInversion.ISINId = paramISINId) AS ValorComprasEjec,
		  (SELECT CASE COUNT(*)
				  WHEN 0 THEN 0
				  ELSE  sum(cantidad * EjecucionPrecio)	/ COUNT(*)
          END
			FROM OrdenInversion
			WHERE TipoOrdenId = 'C' AND
				  EstadoId = 'E' AND
				  OrdenInversion.ISINId = paramISINId) AS ValorMedioComprasEjec,

		 (SELECT COUNT(*)
			FROM OrdenInversion
			WHERE TipoOrdenId = 'V' AND
				  EstadoId = 'E'AND
				  OrdenInversion.ISINId = paramISINId ) AS TotalVentasEjec	,
		 (SELECT CASE COUNT(*)
				  WHEN 0 THEN 0
				  ELSE sum(cantidad * EjecucionPrecio)
          END
		 	FROM OrdenInversion
			WHERE TipoOrdenId = 'V' AND
				  EstadoId = 'E' AND OrdenInversion.ISINId = paramISINId) AS ValorVentasEjec
 INTO   paramISINId, paramTicker, paramTickerDescripcion, paramTotalComprasEjec,
		paramValorComprasEjec , paramValorMedioComprasEjec,
        paramTotalVentasEjec, paramValorVentasEjec
 FROM OrdenInversion
 LEFT JOIN Accion ON
			OrdenInversion.ISINId = Accion.ISINId
 WHERE EXTRACT(YEAR FROM FechaIntroduccion) = paramYear AND
       OrdenInversion.ISINId = paramISINId
 Group By  OrdenInversion.ISINId,  Accion.Ticker, Accion.TickerDescripcion;

END Cons_3_1_Acciones_Anual;

/* c. Donat un any concret com a paràmetre d’entrada: el llistat
	  de totes les accions sobre les quals s’hagi realitzat alguna
	  operació, incloent-hi per a cada acció:
		-IdentIFicador de l’acció.
		-Nom complet de l’acció.
		-Nombre d’operacions de compra executades durant l’any.
		-Suma total en euros de les ordres de compra executades durant l’any sobre l’acció.
		-Valor mitjà en euros de les ordres de compra executades durant l’any sobre l’acció.
		-Nombre d’operacions de venda executades durant l’any sobre l’acció.
		-Suma total en euros de les ordres de venda executades sobre l’acció.
		-Revalorització de l’acció en percentatge des d’1 de gener de l’any indicat. */    
PROCEDURE Cons_3_0_Acciones_Anual
(
	paramYear					IN NUMBER
 )
AS

  vTicker						 Accion.Ticker%TYPE;
  vTickerDescripcion			 Accion.TickerDescripcion%TYPE;
  vTotalComprasEjec				 NUMBER;
  vValorComprasEjec				 NUMBER;
  vValorMedioComprasEjec		 NUMBER;
  vTotalVentasEjec				 NUMBER;
  vValorVentasEjec				 NUMBER;
  vRevalorizacion 				 NUMBER;
	-- Cursor que retorna los diferentes Acciones en OrdenesInversion de un año.
	CURSOR c_accion IS
	SELECT 	dIStinct(ISINId) AS ISINId
	FROM OrdenInversion
	ORDER BY ISINId;

	r_accion c_accion%ROWTYPE;

	BEGIN

	DBMS_OUTPUT.ENABLE;
	
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	DBMS_OUTPUT.PUT_LINE('c. Donat un any concret com a paràmetre d’entrada: el llistat');
	DBMS_OUTPUT.PUT_LINE('de totes les accions sobre les quals s’hagi realitzat alguna');
	DBMS_OUTPUT.PUT_LINE('operació, incloent-hi per a cada acció:');
	DBMS_OUTPUT.PUT_LINE('-Identificador de l’acció.');
	DBMS_OUTPUT.PUT_LINE('-Nom complet de l’acció.');
	DBMS_OUTPUT.PUT_LINE('-Nombre d’operacions de compra executades durant l’any.');
	DBMS_OUTPUT.PUT_LINE('-Suma total en euros de les ordres de compra executades durant l’any sobre l’acció.');
	DBMS_OUTPUT.PUT_LINE('-Valor mitjà en euros de les ordres de compra executades durant l’any sobre l’acció.');
	DBMS_OUTPUT.PUT_LINE('-Nombre d’operacions de venda executades durant l’any sobre l’acció.');
	DBMS_OUTPUT.PUT_LINE('-Suma total en euros de les ordres de venda executades sobre l’acció.');
	DBMS_OUTPUT.PUT_LINE('-Revalorització de l’acció en percentatge des d’1 de gener de l’any indicat. ');
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++');
    DBMS_OUTPUT.PUT_LINE('ACCIONES EN CARTERA DEL AÑO: '||paramYear);
    DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++');
   

	-- Se abre el cursor de acciones
	OPEN c_accion;
	FETCH c_accion INTO r_accion;
    IF c_accion%FOUND THEN
        CLOSE c_accion;
        FOR i in c_accion LOOP

	Cons_3_1_Acciones_Anual(paramYear , i.ISINId, vTicker, vTickerDescripcion,
								  vTotalComprasEjec , vValorComprasEjec,
								  vValorMedioComprasEjec, vTotalVentasEjec, vValorVentasEjec);
		vRevalorizacion := Utilidades.Revalorizacion_Accion_Anual(i.ISINId, paramYear);
          -- Si el cursor retorna regIStros se escriben en el buffer del output.
			DBMS_OUTPUT.PUT_LINE('Accion ...........................: '|| i.ISINId || ' - ' || vTicker || ' - ' || vTickerDescripcion);
			DBMS_OUTPUT.PUT_LINE('Compras Ejecutadas ...............: '|| vTotalComprasEjec);
			DBMS_OUTPUT.PUT_LINE('Valor Compras Ejecutadas .........: '|| ROUND(vValorComprasEjec,2));
			DBMS_OUTPUT.PUT_LINE('Valor Medio Compras Ejecutadas ...: '|| ROUND(vValorMedioComprasEjec,2));
			DBMS_OUTPUT.PUT_LINE('Ventas Ejecutadas ................: '|| vTotalVentasEjec);
			DBMS_OUTPUT.PUT_LINE('Valor Ventas Ejecutadas ..........: '|| ROUND(vValorVentasEjec,2));
			DBMS_OUTPUT.PUT_LINE('Revalorización ...................: '|| ROUND(vRevalorizacion,2) || ' %');
			DBMS_OUTPUT.PUT_LINE(CHR(10));
			DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
			
	    END LOOP;
    ELSE
			DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
			DBMS_OUTPUT.PUT_LINE('LA CONSULTA NO HA DEVUELTO RESULTADOS EN LA CARTERA DE ESTE AÑO.');
			DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
			
	END IF;
  END Cons_3_0_Acciones_Anual;

/* d. LlIStat de les 10 accions més adquirides pels sistemes
	  automàtics, ordenades de forma decreixent pel seu volum. */

/*  Creamos el procedimiento parametrizado y con valor por defecto 
	igual a 10, que es la cantidad que se nos pide en los requerimientos.
	Con la paremtrización ganamos que en un futuro se nos pueda pedir un 
	numero mayor o menor de acciones, con lo cual no deberemos realizar 
	cambios en la programación para poder obtener los nuevos resultados 
	requeridos.	*/  
PROCEDURE Cons_4_Top10Accion
(
  paramTop    in number default 10
) AS
-- Cursor que retorna la cartera de un cliente a una fecha  según los parametros introducidos.
CURSOR c_Top10Accion IS

	SELECT 	OrdenInversion.ISINId , Ticker, TickerDescripcion,
			SUM(Cantidad * SignoTipoOrden) AS Volumen
	FROM OrdenInversion
	LEFT JOIN Accion ON
			OrdenInversion.ISINId = Accion.ISINId
	WHERE 	TIPOORDENID = 'C'    AND
        	TIPOORIGENID = 'A'   AND
			EstadoId = 'E'     
	GROUP BY OrdenInversion.ISINId, Ticker, TickerDescripcion
	ORDER BY Volumen DESC;

r_Top10Accion c_Top10Accion%ROWTYPE;
vlineas Number;
BEGIN
DBMS_OUTPUT.ENABLE;

	DBMS_OUTPUT.PUT_LINE(CHR(10));
	DBMS_OUTPUT.PUT_LINE('d. Llistat de les 10 accions més adquirides pels sistemes');
	DBMS_OUTPUT.PUT_LINE('automàtics, ordenades de forma decreixent pel seu volum.');
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++');
    DBMS_OUTPUT.PUT_LINE('TOP ' || paramTop || ' COMPRAS ACCIONES AUTOMÁTICAS.');
	DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++');
  vlineas := 0;
	-- Se abre el cursor con los datos de la cartera
	OPEN c_Top10Accion;
	FETCH c_Top10Accion INTO r_Top10Accion;
    IF c_Top10Accion%FOUND THEN
        CLOSE c_Top10Accion;
        FOR i IN c_Top10Accion LOOP
		vlineas := vlineas + 1;
          -- Si el cursor retorna regitros es s'escriuen al buffer de l'output.
			DBMS_OUTPUT.PUT_LINE(vlineas ||' - Acción: '|| i.Ticker || ' - ' || i.ISINId || ' - ' || i.TickerDescripcion);
			DBMS_OUTPUT.PUT_LINE('Volumen: '||i.Volumen);
			DBMS_OUTPUT.PUT_LINE(CHR(10));
			DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
	    
		 EXIT WHEN vlineas = paramTop;
		
		END LOOP;
    ELSE
			DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
			DBMS_OUTPUT.PUT_LINE('LA CONSULTA NO HA DEVUELTO RESULTADOS.');
			DBMS_OUTPUT.PUT_LINE('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
			
	END IF;

  END Cons_4_Top10Accion;

END CONSULTAS;
/