--------------------------------------------------------
-- Archivo creado  - mircoles-abril-29-2015   
--------------------------------------------------------
--------------------------------------------------------
--  DDL for Package Body ESTADISTICAS
--------------------------------------------------------

CREATE OR REPLACE PACKAGE BODY "CARTERAINVERSION_USER"."ESTADISTICAS" AS

/* ESTADISTICA 1

 Donat lidentificador dun client i un mes concret (anterior a lactual):
- benefici net en euros i en percentatge de la seva cartera d'accions durant aquell mes. */
PROCEDURE Est_1_Cliente
(
  paramClienteId	IN ClienteHistorico.ClienteId%TYPE,
  paramMes		 	IN NUMBER,
  paramAno		 	IN NUMBER
) AS
	v_year			NUMBER;
	v_mes			NUMBER;
	v_mes_Texto 	VARCHAR(20);
	v_error 		BOOLEAN;
	vBeneficio ClienteHistorico.Beneficio%TYPE; 
	vPorcentaje ClienteHistorico.Porcentaje%TYPE;
	vClienteId ClienteHistorico.ClienteId%TYPE;
	vNombre Cliente.Nombre%TYPE; 

BEGIN	
	v_year := EXTRACT(YEAR FROM SYSDATE); 
	v_mes := EXTRACT(MONTH FROM SYSDATE); 
	v_error := FALSE;	
	/* Comprobacin de posibles errores. */
	IF paramAno > v_year OR paramAno < 2000 THEN 
		v_error := TRUE;
	END IF;	
	IF paramMes > 12 Or paramMes < 1 THEN 
		v_error := TRUE;
	END IF;	
	IF paramAno = v_year AND paramMes >= v_mes THEN 
		v_error := TRUE;
	END IF;	
	/* Obtenemos el nombre del mes */
	SELECT TO_CHAR(TO_DATE(paramMes, 'MM'), 'MONTH') INTO v_mes_Texto  FROM DUAL;
	
	/* Si no hay errores en los parametros de entrada realizamos la consulta */
	IF NOT v_error THEN
		SELECT Beneficio, Porcentaje, ClienteHistorico.ClienteId, Nombre INTO
			vBeneficio, vPorcentaje, vClienteId, vNombre 
		FROM ClienteHistorico 
		INNER JOIN Cliente	ON 
				ClienteHistorico.ClienteId = Cliente.ClienteId
		WHERE ClienteHistorico.ClienteId = paramClienteId AND
				ClienteHistorico.Mes = paramMes AND
				ClienteHistorico.Ano = paramAno;
	END IF;			
	DBMS_OUTPUT.ENABLE; 
	
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	DBMS_OUTPUT.PUT_LINE('ESTADISTICA 1');
	DBMS_OUTPUT.PUT_LINE('Donat l identificador d un client i un mes concret (anterior a l actual):');
	DBMS_OUTPUT.PUT_LINE('benefici net en euros i en percentatge de la seva cartera d accions durant aquell mes.');
	dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('Beneficio neto euros y porcentaje cartera de : ' || vClienteId || ' - ' || vNombre) ;
    dbms_output.put_line('                                           a : ' || v_mes_Texto || ' de ' || paramAno);
    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.new_line;	

	IF NOT v_error THEN
		dbms_output.put_line('Beneficio Neto  ..: '|| ROUND(vBeneficio,2));
		dbms_output.put_line('Porcentaje ........: '|| ROUND(vPorcentaje,2) || ' %');
		DBMS_OUTPUT.PUT_LINE(CHR(10));
		dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
	ELSE	
		dbms_output.put_line('Error en los parametros introducidos');
		dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
	END IF;	
	
END Est_1_Cliente;

/* ESTADISTICA 2

 Donat lidentificador d'una acci:  
- el nombre daccions disponibles dins del broker en aquell mateix moment (s a dir la suma de      totes les accions que estiguin en ordres de venda que no estiguin executades ni cancellades) 
- el nombre de clients que la tenen actualment en la seva cartera de forma activa. */
PROCEDURE Est_2_Disponibilidad_Accion
(
  paramISINId	IN Accion.ISINId%TYPE
) AS
  vISINId Accion.ISINId%TYPE;
  vTicker Accion.Ticker%TYPE;
  vNumeroClientesPoseedores Accion.NumeroClientesPoseedores%TYPE;
  vNumeroAccionesDisponibles Accion.NumeroAccionesDisponibles%TYPE;
  vTickerDescripcion Accion.TickerDescripcion%TYPE;
  BEGIN

	SELECT ISINId,Ticker, NumeroClientesPoseedores, NumeroAccionesDisponibles , TickerDescripcion  INTO
		vISINId, vTicker, vNumeroClientesPoseedores, vNumeroAccionesDisponibles , vTickerDescripcion 
	FROM Accion       
	WHERE ISINId = paramISINId;
	
	DBMS_OUTPUT.ENABLE; 
	
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	DBMS_OUTPUT.PUT_LINE('ESTADISTICA 2');
	DBMS_OUTPUT.PUT_LINE('Donat l identificador d una acci:');
	DBMS_OUTPUT.PUT_LINE('- el nombre d accions disponibles dins del broker en aquell mateix moment s a dir');
	DBMS_OUTPUT.PUT_LINE('  la suma de totes les accions que estiguin en ordres de venda que no estiguin executades ni cancellades ');
	DBMS_OUTPUT.PUT_LINE('  el nombre de clients que la tenen actualment en la seva cartera de forma activa.');
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('Acciones disponible y nmero de clientes poseedores de : ' || vISINId || ' - ' || vTicker  || ' - ' || vTickerDescripcion );
    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('Nmero Acciones disponibles ..: '|| vNumeroAccionesDisponibles);
	dbms_output.put_line('Nmero Clientes poseedores ...: '|| vNumeroClientesPoseedores);
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
	

END Est_2_Disponibilidad_Accion;

/* ESTADISTICA 3

El client amb la cartera daccions amb un valor ms gran, juntament amb el valor en euros de la seva cartera. */
PROCEDURE Est_3_Cliente_Mayor_Cartera AS
  
  vValorCarteraInversiones Est_3_MayorCarteraCliente.ValorCarteraInversiones%TYPE;
  vClienteId Est_3_MayorCarteraCliente.ClienteId%TYPE;
  vNombre Est_3_MayorCarteraCliente.Nombre%TYPE;
  
  BEGIN

	SELECT ValorCarteraInversiones, ClienteId, Nombre INTO
	vValorCarteraInversiones, vClienteId, vNombre
	FROM Est_3_MayorCarteraCliente 
	WHERE Id = 1;
	DBMS_OUTPUT.ENABLE; 
	
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	DBMS_OUTPUT.PUT_LINE('ESTADISTICA 3');
	DBMS_OUTPUT.PUT_LINE('El client amb la cartera d accions amb un valor ms gran, juntament amb el valor en euros de la seva cartera. ');
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('Cliente con la cartera de acciones ms grande: ' || vClienteId || ' - ' || vNombre);
    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('Valor Cartera ..: '|| ROUND(vValorCarteraInversiones,2));
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');

END Est_3_Cliente_Mayor_Cartera;

/* ESTADISTICA 4

Acci ms negociada (comprada i venuda) en el broker de forma histrica juntament amb el seu volum. */
PROCEDURE Est_4_Accion_Mas_Negociada AS
  
  vISINId Est_4_MayorVolumenAccion.ISINId%TYPE;
  vTicker Est_4_MayorVolumenAccion.Ticker%TYPE;
  vVolumenNegociacion Est_4_MayorVolumenAccion.VolumenNegociacion%TYPE;
  vTickerDescripcion Est_4_MayorVolumenAccion.TickerDescripcion%TYPE;
  
  BEGIN

	SELECT ISINId, Ticker, VolumenNegociacion, TickerDescripcion  INTO
		vISINId, vTicker, vVolumenNegociacion, vTickerDescripcion 
	FROM Est_4_MayorVolumenAccion 
	WHERE Id = 1;
	DBMS_OUTPUT.ENABLE; 
  
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	DBMS_OUTPUT.PUT_LINE('ESTADISTICA 4');
	DBMS_OUTPUT.PUT_LINE('Acci ms negociada (comprada i venuda) en el broker de forma histrica juntament amb el seu volum.');
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('Accin ms negociada: ' || vISINId || ' - ' || vTicker || ' - ' || vTickerDescripcion);
    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('Volumen de negociacin ..: '|| vVolumenNegociacion);
	DBMS_OUTPUT.PUT_LINE(CHR(10));
	dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
	
END Est_4_Accion_Mas_Negociada;

END ESTADISTICAS;

/
