﻿
DROP SEQUENCE SEQ_Cliente; 
DROP SEQUENCE SEQ_Cartera; 
DROP SEQUENCE SEQ_OrdenInversion; 
DROP SEQUENCE SEQ_OrdenTrading; 
DROP SEQUENCE SEQ_Log; 

CREATE  SEQUENCE SEQ_Cliente
  MINVALUE 1
  START WITH 1
  INCREMENT BY 1
  CACHE 20;
  
CREATE SEQUENCE SEQ_Cartera
  MINVALUE 1
  START WITH 1
  INCREMENT BY 1
  CACHE 20;  

CREATE  SEQUENCE SEQ_OrdenInversion  MINVALUE 1
  START WITH 1
  INCREMENT BY 1
  CACHE 20;

CREATE  SEQUENCE SEQ_OrdenTrading
  MINVALUE 1
  START WITH 1
  INCREMENT BY 1
  CACHE 20;
  
CREATE SEQUENCE SEQ_Log
  MINVALUE 1
  START WITH 1
  INCREMENT BY 1
  CACHE 20;
  
  
SPOOL Resultados_Instalacion_Test\50_Creacion_Secuencias_Triggers.log; 
---- Utilizaremos el procedimiento RAISE_APPLICATION_ERROR para comunicar 
---- de forma interactiva una excepción predefinida, devolviendo un 
---- código y un mensaje de error no estándar.
--

CREATE  OR REPLACE TRIGGER insCliente  BEFORE INSERT OR UPDATE ON Cliente
FOR EACH ROW
DECLARE
   IsOK boolean;
BEGIN

IF Utilidades.ValidaNIFCIF(:new.NIF) = False THEN
	 RAISE_APPLICATION_ERROR (-20010, 'Error NIF');
END IF;

IF :new.ClienteId IS NULL THEN
	SELECT SEQ_Cliente.NEXTVAL INTO :NEW.ClienteId
	FROM DUAL;
END IF;

-- normalizar formato de los datos introducidos
  :NEW.Nombre := UPPER(TRIM(:new.Nombre));
  :NEW.NIF := UPPER(TRIM(:new.NIF));
  :NEW.Email := LOWER(TRIM(:new.Email));
  :NEW.EstaActivo := UPPER(TRIM(:new.EstaActivo));
END insCliente ;
/


CREATE OR REPLACE
TRIGGER insCartera BEFORE INSERT OR UPDATE ON Cartera
FOR EACH ROW
BEGIN
IF :new.CarteraId IS NULL THEN
	SELECT SEQ_Cartera.NEXTVAL INTO :NEW.CarteraId
	FROM DUAL;
END IF;
  -- normalizar formato de los datos introducidos
  :NEW.CarteraDescripcion := UPPER(TRIM(:new.CarteraDescripcion));
  :NEW.EstaActivo := UPPER(TRIM(:new.EstaActivo));
END insCartera ;
/


CREATE OR REPLACE
TRIGGER insOrdenInversion BEFORE INSERT OR UPDATE ON OrdenInversion
FOR EACH ROW
BEGIN
IF :new.OrdenInversionId IS NULL THEN
	SELECT SEQ_OrdenInversion.NEXTVAL INTO :NEW.OrdenInversionId
	FROM DUAL;
END IF;

IF(:old.EstadoId <> 'P') THEN
 RAISE_APPLICATION_ERROR (-20000, 'Orden de Inversión no puede ser modificada');
END IF ;

END insOrdenInversion ;
/

CREATE OR REPLACE
TRIGGER insOrdenTrading BEFORE INSERT OR UPDATE ON OrdenTrading
FOR EACH ROW
BEGIN
IF :new.OrdenTradingId IS NULL THEN
	SELECT SEQ_OrdenTrading.NEXTVAL INTO :NEW.OrdenTradingId
	FROM DUAL;
END IF;
END insOrdenTrading ;
/


CREATE OR REPLACE
TRIGGER insLog BEFORE INSERT  ON Log
FOR EACH ROW
BEGIN
IF :new.ProcesoId IS NULL THEN
	SELECT SEQ_Log.NEXTVAL INTO :NEW.ProcesoId
	FROM DUAL;
END IF;
END insLog ;
/


CREATE OR REPLACE
TRIGGER insAccion BEFORE INSERT  ON Accion
FOR EACH ROW
BEGIN
 -- normalizar formato de los datos introducidos
  :NEW.ISINId := UPPER(TRIM(:new.ISINId));
  :NEW.Ticker := UPPER(TRIM(:new.Ticker));
  :NEW.TickerDescripcion := UPPER(TRIM(:new.TickerDescripcion));
END insAccion ;
/

CREATE OR REPLACE
TRIGGER insTrading BEFORE INSERT  ON Trading
FOR EACH ROW
BEGIN
 -- normalizar formato de los datos introducidos
  :NEW.TradingId := UPPER(TRIM(:new.TradingId));
  :NEW.TradingDescripcion := UPPER(TRIM(:new.TradingDescripcion));
END insTrading ;
/

CREATE OR REPLACE
TRIGGER insMercado BEFORE INSERT  ON Mercado
FOR EACH ROW
BEGIN
 -- normalizar formato de los datos introducidos
  :NEW.MercadoId := UPPER(TRIM(:new.MercadoId));
  :NEW.MercadoDescripcion := UPPER(TRIM(:new.MercadoDescripcion));
END insMercado ;
/

CREATE OR REPLACE
TRIGGER insTipoOrigen BEFORE INSERT  ON TipoOrigen
FOR EACH ROW
BEGIN
 -- normalizar formato de los datos introducidos
  :NEW.TipoOrigenId := UPPER(TRIM(:new.TipoOrigenId));
  :NEW.TipoOrigenDescripcion := UPPER(TRIM(:new.TipoOrigenDescripcion));
END insTipoOrigen ;
/

CREATE OR REPLACE
TRIGGER insTipoOrden BEFORE INSERT  ON TipoOrden
FOR EACH ROW
BEGIN
 -- normalizar formato de los datos introducidos
  :NEW.TipoOrdenId := UPPER(TRIM(:new.TipoOrdenId));
  :NEW.TipoOrdenDescripcion := UPPER(TRIM(:new.TipoOrdenDescripcion));
END insTipoOrden ;
/

CREATE OR REPLACE
TRIGGER insOrdenEstado BEFORE INSERT  ON OrdenEstado
FOR EACH ROW
BEGIN
 -- normalizar formato de los datos introducidos
  :NEW.EstadoId := UPPER(TRIM(:new.EstadoId));
  :NEW.EstadoOrdenDescripcion := UPPER(TRIM(:new.EstadoOrdenDescripcion));
END insTipoOrden ;
/


-- Trigger para mantener el valor de la cartera de cada cliente
create or replace TRIGGER UPDSITUACIONCARTERA
AFTER UPDATE  ON SITUACIONCARTERA
FOR EACH ROW
DECLARE
valor 						NUMBER;
valorMes     				NUMBER;
vANO 						NUMBER;
vMes 						NUMBER;
vANOAnterior 				NUMBER;
vMesAnterior				NUMBER;
existe						NUMBER;
vClienteId					NUMBER;
vNombre						VARCHAR2(150);
vValorCarteraInversiones	NUMBER;
newValorCarteraInversiones	NUMBER;
oldValorCarteraInversiones	NUMBER;
vISINId						CHAR(12);
vTicker						VARCHAR2(10);
vTickerDescripcion			VARCHAR2(100);
vVolumenNegociacion			NUMBER;
RSP  						VARCHAR2(50);
BEGIN

/* Se actualiza el valor cartera inversiones de la tabla cliente con el diferencial
	- entre el diferencial de la cantidad anterior y la nueva cantidad.
	- entre el valor de cotización anterior y el nuevo valor */

	newValorCarteraInversiones := 0;
	oldValorCarteraInversiones := 0;
IF((:new.cantidad <>  :old.cantidad ) OR (:new.ValorCotizacionUlt <> :old.ValorCotizacionUlt)) THEN

	SELECT ValorCarteraInversiones INTO oldValorCarteraInversiones
	FROM Cliente
	WHERE ClienteId = :new.ClienteId;

	valor := (:new.cantidad * :new.ValorCotizacionUlt)- (:old.cantidad * :old.ValorCotizacionUlt) ;
	update Cliente
	set ValorCarteraInversiones =  ValorCarteraInversiones+valor
    where ClienteId = :new.ClienteId;

	SELECT ValorCarteraInversiones INTO newValorCarteraInversiones
	FROM Cliente
	WHERE ClienteId = :new.ClienteId;
	

	SELECT ValorCarteraInversiones, ClienteId, Nombre INTO vValorCarteraInversiones, vClienteId, vNombre
	FROM (SELECT ValorCarteraInversiones, ClienteId, Nombre
		  FROM Cliente
		  ORDER BY ValorCarteraInversiones DESC)
	WHERE ROWNUM = 1;

	/* La operación UPSERT actualiza o inserta una fila en una tabla,
	dependiendo de si la tabla ya tiene una fila que coincide con los
	datos.
	Ya que Oracle no tiene una declaración específica de UPSERT,
	utilizaremos la declaración "merge" que fusiona los datos
	entre dos tablas. Uso DUAL nos permite utilizar este comando. */

	merge into Est_3_MayorCarteraCliente m
	using dual on (Id = 1)
	when not matched then
	insert (Id,ClienteId, Nombre, ValorCarteraInversiones)
		values (1, vClienteId, vNombre, vValorCarteraInversiones)
	when matched then
	update set 	ClienteId = vClienteId,
				Nombre = vNombre,
				ValorCarteraInversiones = vValorCarteraInversiones;
END IF ;

/* 	Se actualiza el volumen de negociación de la acción.
	Se tiene en cuenta el diferencial de la cantidad anterior
	respecto a la nueva cantidad en valor absoluto.*/

IF (:new.cantidad <>  :old.cantidad ) THEN
	valor := ABS((:new.cantidad )- (:old.cantidad )) ;
	update Accion set VolumenNegociacion =  VolumenNegociacion+valor
    where ISINId = :new.ISINId;


	SELECT ISINId,Ticker, VolumenNegociacion, TickerDescripcion INTO vISINId, vTicker, vVolumenNegociacion, vTickerDescripcion
	FROM (SELECT ISINId,Ticker, VolumenNegociacion, TickerDescripcion
		  FROM Accion
		  ORDER BY VOLUMENNEGOCIACION DESC)
	WHERE ROWNUM = 1;

	/* La operación UPSERT actualiza o inserta una fila en una tabla,
	dependiendo de si la tabla ya tiene una fila que coincide con los
	datos.
	Ya que Oracle no tiene una declaración específica de UPSERT,
	utilizaremos la declaraciÃ³n "merge" que fusiona los datos
	entre dos tablas. Uso DUAL nos permite utilizar este comando. */

	merge into Est_4_MayorVolumenAccion m
	using dual on (Id = 1)
	when not matched then
	insert (Id,ISINId, Ticker, VolumenNegociacion, TickerDescripcion)
		values (1, vISINId, vTicker, vVolumenNegociacion, vTickerDescripcion)
	when matched then
	update set 	ISINId = vISINId,
				Ticker = vTicker,
				VolumenNegociacion = vVolumenNegociacion,
				TickerDescripcion = vTickerDescripcion;

END IF ;

/* Se actualiza la cantidad pendiente de venta de la acción.
	Se tiene en cuenta el diferencial de la cantidad pendiente anterior
	respecto a la nueva cantidad pendiente. */
IF (:new.CantidadPendienteVenta <>  :old.CantidadPendienteVenta ) THEN
 valor := (:new.CantidadPendienteVenta )- (:old.CantidadPendienteVenta ) ;
 update Accion set NumeroAccionesDisponibles =  NumeroAccionesDisponibles + valor
    where ISINId = :new.ISINId;
END IF ;

/* Si se produce un cambio de mes se generara ClienteHistorico   */
IF (:new.FechaCotizacionUlt  <>  :old.FechaCotizacionUlt ) THEN
 vANO := EXTRACT(YEAR FROM :new.FechaCotizacionUlt);
 vMes := EXTRACT(MONTH FROM :new.FechaCotizacionUlt);

 SELECT count(*) INTO existe
	FROM ClienteHistorico
	WHERE ClienteId = :new.ClienteId;
	IF (existe = 0) THEN /* La primera vez se carga los dos valores iguales */
	oldValorCarteraInversiones:= newValorCarteraInversiones;
	END IF;  
		  
SELECT count(*) INTO existe
	FROM ClienteHistorico
	WHERE ANO = vANO AND
		  Mes = vMes AND
		  ClienteId = :new.ClienteId;		  
		
	IF (existe = 0) THEN /* se genera el nuevo registro del mes */
		/* miramos que exista el mes anterior */
		vANOAnterior := vANO;
		vMesAnterior := vmes -1;
		IF (vMesAnterior = 0) THEN
			vMesAnterior := 12;
			vANOAnterior := vANO - 1;
		END IF;

		SELECT count(*) INTO existe 
		FROM ClienteHistorico
		WHERE ANO = vANOAnterior AND
			  Mes = vMesAnterior AND
			  ClienteId = :new.ClienteId ;

		IF (existe > 0) THEN 
		/* Si existe recogemos el valor final del mes anterior */
			SELECT  IMPORTEFINAL INTO  valorMes
			  FROM ClienteHistorico
			  WHERE ANO = vANOAnterior AND
				Mes = vMesAnterior AND
				ClienteId = :new.ClienteId ;
			oldValorCarteraInversiones:= valorMes; 
		END IF;
		/* En este punto oldValorCarteraInversiones tendra el valor 
		   de newValorCarteraInversiones si es la primera vez que se carga este cliente
		   o bien tendra el valor ImporteFinal del mes anterior.		*/
		GestionTest.ins_ClienteHistorico(:new.ClienteId, vMes, vANO, oldValorCarteraInversiones, newValorCarteraInversiones, RSP );
	ELSE 
	/* Durante el mes se actualiza el valor en ImporteFinal*/	
		SELECT  IMPORTEINICIAL INTO  valorMes
		FROM ClienteHistorico
		WHERE ANO = vANO AND
			  Mes = vMes AND
			  ClienteId = :new.ClienteId ;
		GestionTest.upd_ClienteHistorico(:new.ClienteId, vMes, vANO, valorMes, newValorCarteraInversiones, RSP );
		
	END IF;

END IF;
END;

/
	
CREATE OR REPLACE
TRIGGER insBEFOREAccionHistorico BEFORE INSERT  ON ACCIONHISTORICO
FOR EACH ROW
DECLARE
maximaFechaCierre date;
BEGIN

SELECT MAX(FechaCierre) INTO maximaFechaCierre
	FROM AccionHistorico; 
	
	/* no podemos pasar cotizaciones anteriores a la última fecha introducida */
	IF (maximaFechaCierre > :new.FechaCierre) THEN
		RAISE_APPLICATION_ERROR (-20020, 'La fecha de cierre ' || :new.FechaCierre || 
								' es inferior a la última introducida ' || maximaFechaCierre );
	END IF;
END insBEFOREAccionHistorico ;
/	

CREATE OR REPLACE TRIGGER INSACCIONHISTORICO 
AFTER INSERT ON ACCIONHISTORICO 
FOR EACH ROW
DECLARE
BEGIN 
	UPDATE SituacionCartera SET ValorCotizacionUlt = :new.ValorCierre, FechaCotizacionUlt = :new.FechaCierre
    WHERE ISINID = :new.ISINId;
END;
/
SPOOL OFF;
